package com.gct.aoid.algorithm;

import com.alibaba.fastjson.JSONObject;
import com.gct.common.util.SGTUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * class name: CalGTCorrect
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/7/23 上午10:35 Fri
 */
public class CalGTCorrect {


    public static List<Double[]> calSlopePointsArray(List<Double[]> points) {

        if (points == null || points.size() < 2) return null;

        //deep copy head node and append tail
        points.add(new Double[]{points.get(0)[0], points.get(0)[1]});

        List<Double[]> lis = new ArrayList<>();
        int xSamePointNum = 0;
        for (int i = 0; i < points.size() - 1; i++) {
            Double deltaX = points.get(i + 1)[0] - points.get(i)[0];

            //region fun 2:remove the point when deltaX=0-----------(Deprecated)
            /* if(deltaX==0) {
                points.remove(i);
                i--;
                continue;
            }*/

            //region fun 2:use absolute val=0.001~0.009 when deltaX=0-----------(Deprecated)
            //if (deltaX == 0) deltaX = 0.009;
            //endregion

            //region fun 3:use line fill x ---------------------Recommended
            if (deltaX == 0) {
                xSamePointNum++;
                continue;
            } else {
                if (xSamePointNum != 0) {
                    double stepX = deltaX / (xSamePointNum + 1);
                    for (int j = xSamePointNum - 1; j >= 0; j--) {
                        points.get(i - j)[0] = points.get(i - xSamePointNum)[0] + (xSamePointNum - j) * stepX;
                    }
                    i = i - xSamePointNum;
                    i--;
                    xSamePointNum = 0;
                    continue;
                }
            }
            //endregion

            Double slope = (points.get(i + 1)[1] - points.get(i)[1]) / deltaX;
            slope = BigDecimal.valueOf(slope).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            Double x = (points.get(i)[0] + points.get(i + 1)[0]) / 2;
            x = BigDecimal.valueOf(x).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            lis.add(new Double[]{x, slope});
        }
        return lis;
    }

    /**
     * 位移修正,平移到第一象限
     *
     * @param points
     * @param minS
     */
    public static void strokeCorrect(List<Double[]> points, double minS) {
        if (points == null || points.size() == 0) return;
        if (minS >= 0) return;
        minS = Math.abs(minS);
        for (int i = 0; i < points.size(); i++) {
            points.get(i)[0] = points.get(i)[0] + minS;
        }
    }

    /**
     * 起始点位置修正
     *
     * @param slopeOptimizedPoints
     */
    public static void locateStartPoint(List<Double[]> slopeOptimizedPoints) {
        if (Objects.isNull(slopeOptimizedPoints)) return;
        int startIndex = 0;

        for (int i = 0; i < slopeOptimizedPoints.size(); i++) {
            if (slopeOptimizedPoints.get(startIndex)[0] == 0) break;

            if (slopeOptimizedPoints.get(i)[0] < slopeOptimizedPoints.get(startIndex)[0])
                startIndex = i;
            else if (slopeOptimizedPoints.get(i)[0] == slopeOptimizedPoints.get(startIndex)[0])
                continue;
            else break;
        }
        for (int i = 0; i < startIndex; i++) {
            Double[] temp = slopeOptimizedPoints.get(0);
            slopeOptimizedPoints.remove(0);
            slopeOptimizedPoints.add(temp);
        }
    }

    /**
     * 垂直反转修正
     *
     * @param slopeOptimizedPoints
     * @param slopes
     * @param maxS
     * @param minL
     * @param maxL
     */
    public static void verticalReversalCorrect(List<Double[]> slopeOptimizedPoints, List<Double[]> slopes, Double maxS, Double minL, Double maxL) {

        if (Objects.isNull(slopeOptimizedPoints) || Objects.isNull(slopes) || slopeOptimizedPoints.isEmpty() || slopes.isEmpty())
            return;
        double sLimit = 0.43 * maxS;//   <43%
        double tempArea = 0;
        for (int i = 1; i < slopes.size(); i++) {
            if (i >= slopes.size() / 2) break;
            if (slopes.get(i)[0] <= sLimit && slopes.get(i - 1)[0] <= sLimit)
                tempArea = tempArea +
                        (slopes.get(i)[1] + slopes.get(i - 1)[1]) * (slopes.get(i)[0] - slopes.get(i - 1)[0]) / 2;
            else break;
        }

        if (tempArea >= 0) return;

        for (int i = 0; i < slopeOptimizedPoints.size(); i++)
            slopeOptimizedPoints.get(i)[1] = minL + maxL - slopeOptimizedPoints.get(i)[1];
    }

    @Deprecated
    public static void verticalReversalCorrect(List<Double[]> slopeOptimizedPoints, Double maxS, Double minL, Double maxL) {
        if (Objects.isNull(slopeOptimizedPoints) || slopeOptimizedPoints.isEmpty())
            return;
//        double sLimit = 0.43 * maxS;//   <43%
        double upLoad = 0;
        double downLoad = 0;
        for (int i = 0; i < slopeOptimizedPoints.size() / 2; i++) {
            upLoad += slopeOptimizedPoints.get(i)[1];
            downLoad += slopeOptimizedPoints.get(slopeOptimizedPoints.size() - i - 1)[1];
        }
        if (downLoad <= upLoad) return;

        for (int i = 0; i < slopeOptimizedPoints.size(); i++)
            slopeOptimizedPoints.get(i)[1] = minL + maxL - slopeOptimizedPoints.get(i)[1];

    }

    @Deprecated
    public static List<Double[]> calSlopePointsArray_Old(List<Double[]> points) {

        if (points == null || points.size() < 2) return null;

        //deep copy head node and append tail
        points.add(new Double[]{points.get(0)[0], points.get(0)[1]});

        List<Double[]> lis = new ArrayList<>();
        int xSamePointNum = 0;
        for (int i = 0; i < points.size() - 1; i++) {
            Double deltaX = points.get(i + 1)[0] - points.get(i)[0];

            //region fun 2:remove the point when deltaX=0-----------(Deprecated)
            if (deltaX == 0) {
                points.remove(i);
                i--;
                continue;
            }

            //region fun 2:use absolute val=0.001~0.009 when deltaX=0-----------(Deprecated)
            //if (deltaX == 0) deltaX = 0.009;
            //endregion

            //region fun 3:use line fill x ---------------------Recommended
           /* if (deltaX == 0) {
                xSamePointNum++;
                continue;
            } else {
                if (xSamePointNum != 0) {
                    double stepX = deltaX / (xSamePointNum + 1);
                    for (int j = xSamePointNum - 1; j >= 0; j--) {
                        points.get(i - j)[0] = points.get(i - xSamePointNum)[0] + (xSamePointNum - j) * stepX;
                    }
                    i = i - xSamePointNum;
                    i--;
                    xSamePointNum = 0;
                    continue;
                }
            }*/
            //endregion

            Double slope = (points.get(i + 1)[1] - points.get(i)[1]) / deltaX;
            slope = BigDecimal.valueOf(slope).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            Double x = (points.get(i)[0] + points.get(i + 1)[0]) / 2;
            x = BigDecimal.valueOf(x).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
            lis.add(new Double[]{x, slope});
        }
        return lis;
    }


}
