package com.gct.aoid.algorithm;

import java.util.ArrayList;
import java.util.List;

/**
 * @author xusirui 2021/6/9
 */
public class CurveFitting {
    /**
     * @param arrX x值
     * @param arrY y值
     * @param length 数据点数
     * @param dimension 多项式最高次幂
     * @return 返回多项式系数从高到低
     */
    public double[] multiLine(double[] arrX, double[] arrY, int length, int dimension) {
        int k = dimension + 1;
        double[][] gauss = new double[k][k + 1];
        for (int j = 0; j < k; j++) {
            int i = 0;
            for (; i < k; i++) {
                gauss[j][i] = sumArr(arrX, i + j, length);
            }
            gauss[j][i] = sumArr(arrX, j, arrY, 1, length);
        }
        return computeGauss(gauss, k);
    }

    private double sumArr(double[] arr, int n, int length) {
        double s = 0;
        for (int i = 0; i < length; i++) {
            s = arr[i] == 0.0 & n == 0 ? s + 1.0 : s + Math.pow(arr[i], n);
        }
        return s;
    }

    private double sumArr(double[] arr1, int n1, double[] arr2, int n2, int length) {
        double s = 0;
        for (int i = 0; i < length; i++) {
            s = (arr1[i] == 0.0 && n1 == 0) || (arr2[i] == 0.0 && n2 == 0) ? s + 1.0 : s + Math.pow(arr1[i], n1) * Math.pow(arr2[i], n2);
        }
        return s;
    }

    private double[] computeGauss(double[][] guass, int n) {
        double[] x = new double[n];
        for (int i = 0; i < n; i++) x[i] = 0;
        for (int i = 0; i < n; i++) {
            double max = 0;
            int m = i;
            for (int j = i; j < n; j++) {
                if (Math.abs(guass[j][i]) > max) {
                    max = guass[m = j][i];
                }
            }
            if (m != i) {
                for (int j = i; j < n + 1; j++) {
                    double temp = guass[i][j];
                    guass[i][j] = guass[m][j];
                    guass[m][j] = temp;
                }
            }
            if (max == 0.0) return x;
            for (int j = i + 1; j < n; j++) {
                double s = guass[j][i];
                for (int k = i; k < n + 1; k++) {
                    guass[j][k] -= guass[i][k] * s / guass[i][i];
                }
            }
        }
        for (int i = n - 1; i >= 0; i--) {
            double s = 0;
            for (int j = i + 1; j < n; j++) {
                s += guass[i][j] * x[j];
            }
            x[i] = (guass[i][n] - s) / guass[i][i];
        }
        return x;
    }

    public double computeY(double x, double[] fit) {
        double y = 0;
        for (int i = 0; i < fit.length; i++) {
            y += fit[i] * Math.pow(x, i);
        }
        return y;
    }

    /**
     * 计算一阶导数
     * @param x
     * @param fit
     * @return
     */
    public double computeYFirstDerivative(double x,double[] fit){
        double y = 0;
        for (int i = 1; i < fit.length; i++) {
            y += fit[i] * Math.pow(x, i-1)*i;
        }
        return y;
    }

    /**
     * 计算二阶导
     * @param x
     * @param fit
     * @return
     */
    public double computeYSecondDerivative(double x,double[] fit){
        double y = 0;
        for (int i = 2; i < fit.length; i++) {
            y += fit[i] * Math.pow(x, i-2)*i*(i-1);
        }
        return y;
    }

    public double cRotationalCoordinates(double[] arrX, double[] arrY, double[] fit) {
        List<Double> listA = new ArrayList<>();
        List<Double> listB = new ArrayList<>();
        for (int i = 0; i < arrX.length && i < arrY.length; i++) {
            listA.add(arrY[i]);
            listB.add(computeY(arrX[i], fit));
        }
        return pearson(listA, listB);
    }

    private double[] cRotationalCoordinates(double x, double y, double x0, double y0, double a) {
        double[] res = new double[2];
        res[0] = (x - x0) * Math.cos(a) - (y - y0) * Math.sin(a) + x0;
        res[1] = (x - x0) * Math.sin(a) + (y - y0) * Math.cos(a) + y0;
        return res;
    }

    public double pearson(List<Double> listA, List<Double> listB) {
        double varA = 0;
        double varB = 0;
        double aveA = 0;
        double aveB = 0;
        double r = 0;
        for (int i = 0; i < listA.size(); i++) {
            double ai = listA.get(i);
            double bi = listB.get(i);
            varA += (ai - aveA) * (ai - aveA) * i / (i + 1);
            varB += (bi - aveB) * (bi - aveB) * i / (i + 1);
            r += (ai - aveA) * (bi - aveB) * i / (i + 1);
            aveA += (ai - aveA) / (i + 1);
            aveB += (bi - aveB) / (i + 1);
        }
        return r / Math.sqrt(varA * varB);
    }
}
