package com.gct.aoid.algorithm;

import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * dec:
 * <p>
 * EffectStroke
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2022/8/9 下午1:51
 */
public class EffectStroke {

    private static final CurveFitting fitTool = new CurveFitting();
    private static final int unitPointNum = 5;

    /**
     * 计算向量夹角
     *
     * @param vec1 one
     * @param vec2 two
     * @return du
     */
    private static Double vectorAngle(double[] vec1, double[] vec2) {
        if (Objects.isNull(vec1) || Objects.isNull(vec2) || vec1.length != 2 || vec1.length != vec2.length) return 0d;
        double theta = (vec1[0] * vec2[0] + vec1[1] * vec2[1]) / (Math.sqrt(vec1[0] * vec1[0] + vec1[1] * vec1[1]) * Math.sqrt(vec2[0] * vec2[0] + vec2[1] * vec2[1]));
        return Math.acos(theta) / 2 / Math.PI * 360;
    }

    public static Double[] cardCurvatures(Double[][] sgt) {
        if (unitPointNum % 2 == 0 || unitPointNum < 3) throw new RuntimeException("分段点数用奇数且大于等于3");
        if (Objects.isNull(sgt) || sgt.length < unitPointNum) throw new RuntimeException("功图不可用");

        int start = unitPointNum / 2;
        Double[] cts = new Double[sgt.length];
        List<Double[]> unitLine = new ArrayList<>();
        do {
            int unitEnd = start + unitPointNum / 2;
            if (unitEnd >= sgt.length) unitEnd -= sgt.length;
            if (unitLine.isEmpty()) for (int i = 0; i <= unitEnd; i++) unitLine.add(sgt[i]);
            else {
                unitLine.remove(0);
                unitLine.add(sgt[unitEnd]);
            }

            cts[start] = pointCurvatures(unitLine);//计算中间点的曲率

            start++;
            if (start >= sgt.length) start -= sgt.length;
        } while (start != unitPointNum / 2);
        return cts;
    }

    private static Double pointCurvatures(List<Double[]> unitLine) {
        double[] x = new double[unitLine.size()];
        double[] y = new double[unitLine.size()];
        for (int i = 0; i < unitLine.size(); i++) {
            x[i] = unitLine.get(i)[0];
            y[i] = unitLine.get(i)[1];
        }
        //二次曲线拟合
        double[] eo = fitTool.multiLine(x, y, unitLine.size(), 2);
        //x值
        double X = x[x.length / 2];
        //一阶导
        double Y1 = fitTool.computeYFirstDerivative(X, eo);
        //二阶导
        double Y2 = fitTool.computeYFirstDerivative(X, eo);
        //曲率
        return Math.abs(Y2) / Math.pow(1 + Y1 * Y1, 1.5);

    }

    public static void main(String[] args) {
        String sgt = "[[0.0, 63.44], [0.0, 63.44], [0.0, 66.64], [0.0, 71.12], [0.01, 78.8], [0.02, 79.44], [0.02, 85.2], [0.04, 85.84], [0.05, 85.84], [0.07, 85.84], [0.08, 80.08], [0.1, 78.8], [0.13, 76.24], [0.15, 69.84], [0.18, 62.8], [0.21, 62.16], [0.23, 65.36], [0.27, 66.64], [0.3, 64.08], [0.34, 65.36], [0.37, 65.36], [0.42, 76.24], [0.46, 74.96], [0.49, 74.96], [0.54, 71.12], [0.58, 66.0], [0.64, 65.36], [0.67, 65.36], [0.72, 59.6], [0.77, 57.04], [0.81, 56.4], [0.86, 59.6], [0.92, 63.44], [0.97, 64.08], [1.01, 67.92], [1.06, 68.56], [1.12, 69.84], [1.15, 71.12], [1.2, 73.68], [1.26, 70.48], [1.31, 64.72], [1.35, 64.08], [1.4, 63.44], [1.45, 62.8], [1.48, 62.16], [1.54, 62.8], [1.58, 61.52], [1.62, 60.88], [1.72, 24.84], [4.28, 24.4], [3.25, 24.36], [1.2, 24.56], [0.18, 24.64], [14.51, 24.48], [11.44, 24.56], [9.39, 24.56], [6.32, 25.24], [2.22, 25.16], [15.53, 25.16], [11.44, 24.92], [6.32, 24.6], [1.19, 24.56], [13.48, 24.56], [7.34, 24.2], [1.19, 24.04], [12.45, 24.0], [6.3, 24.2], [14.5, 24.44], [6.3, 24.48], [1.18, 24.72], [9.37, 24.76], [0.15, 24.84], [10.39, 24.92], [1.17, 25.08], [7.31, 24.88], [14.48, 24.52], [7.31, 24.48], [12.43, 24.44], [2.18, 24.4], [11.4, 24.36], [0.13, 24.4], [5.25, 24.32], [9.34, 24.28], [2.17, 24.56], [6.26, 24.64], [10.36, 24.72], [2.16, 24.64], [6.26, 24.72], [10.35, 24.76], [2.16, 24.56], [5.22, 24.28], [9.32, 24.12], [12.39, 24.0], [4.19, 24.04], [7.26, 23.96], [10.33, 24.0], [2.14, 23.88], [5.2, 23.96], [8.27, 24.16], [12.37, 24.08], [3.15, 24.0], [7.24, 23.76], [10.31, 22.88], [2.12, 22.8], [5.18, 22.84], [9.28, 22.56], [13.37, 22.52], [5.18, 22.16], [9.27, 22.16], [13.36, 22.12], [5.17, 22.08], [9.26, 22.28], [14.38, 22.12], [2.09, 22.28], [11.3, 19.48], [0.04, 17.48], [5.15, 17.08], [14.37, 17.4], [3.1, 17.76], [10.27, 18.6], [3.1, 18.32], [9.24, 18.4], [0.02, 21.12], [7.19, 22.32], [1.04, 22.24], [8.21, 21.88], [0.01, 20.2], [11.28, 20.0], [4.1, 19.96], [13.32, 18.8], [6.15, 18.04], [2.05, 17.08], [12.29, 17.52], [7.17, 18.04], [4.1, 19.44], [15.36, 19.76], [11.26, 19.92], [8.19, 20.8], [6.14, 21.4], [4.1, 21.36], [2.05, 21.48], [1.17, 20.52], [0.15, 19.36], [8.34, 18.72], [0.15, 18.72], [7.32, 18.04], [11.42, 18.12], [1.18, 18.04], [7.33, 18.28], [9.39, 19.32], [2.22, 20.32], [9.4, 20.56], [3.26, 21.32], [10.42, 21.28], [11.44, 21.04], [11.44, 21.0], [9.39, 20.68], [1.19, 19.88], [9.38, 19.48], [4.26, 19.28], [9.38, 18.72], [6.3, 18.76], [14.5, 19.28], [2.21, 19.6], [13.48, 20.32], [6.31, 21.0], [10.41, 21.2], [7.34, 21.92], [4.26, 22.24], [5.28, 21.44], [8.36, 21.2], [2.21, 20.88], [2.2, 20.0], [7.32, 20.48], [10.39, 20.16], [5.27, 19.6], [1.18, 20.56], [3.22, 21.0], [3.22, 21.32], [8.34, 21.32], [15.52, 22.44], [11.42, 22.52], [12.45, 22.6], [5.28, 22.8], [6.3, 21.88], [6.3, 22.12], [6.3, 22.16], [13.47, 21.48], [11.42, 21.24], [7.82, 21.36], [10.7, 21.28], [10.5, 22.16], [10.57, 22.6], [9.74, 22.88], [9.35, 22.76], [8.33, 23.88], [7.62, 23.84], [6.79, 23.84], [6.47, 23.76], [6.22, 23.48]]";
        Double[][] cardArr = JSONObject.parseObject(sgt, Double[][].class);
        System.out.println(JSONObject.toJSONString(cardCurvatures(cardArr)));

    }
}
