package com.gct.aoid.config;

import com.gct.aoid.job.*;
import com.gct.aoid.job.properties.JobConfigurationProperties;
import com.gct.aoid.job.properties.JobParamConfigurationProperties;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.job.properties.TriggerConfigurationProperties;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;
import java.text.ParseException;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;

/**
 * class name: QuartzTaskConfig
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:21
 */
@Configuration
@EnableConfigurationProperties(MutableJobConfigProperties.class)
@ConditionalOnProperty(prefix = "gct.quartz.jobs.cal-aoid-liq-error.param", name = "enabled", havingValue = "true")
public class LiqAbnormalMonitorConfig {
    String jobName = "calAoidLiqError";
    String jobGroup = "CALAOIDLIQERROR_GROUP";

    /**
     * 量液异常JobDetail
     *
     * @return
     */
    @Bean
    public JobDetail CalAoidLiqErrorJob() {
        System.out.println("---------------------------");
        try {
            return JobBuilder.newJob(CalAoidLiqError.class)
                    .withIdentity(jobName, jobGroup)
                    .storeDurably()
                    .build();
        } catch (Exception e) {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", jobName, jobGroup);
            throw e;
        }

    }

    /**
     * 量液异常
     *
     * @return
     */
    @Bean
    public Trigger CalAoidLiqErrorJobTrigger() throws IOException {
        try {

            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInSeconds(60);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(jobName, jobGroup)
                    .startAt(new Date(new Date().getTime() + 5000))
                    .withSchedule(scheduleBuilder).build();
        } catch (Exception e) {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}", jobName, jobGroup, e.getMessage());
            throw e;
        }


    }
}
