package com.gct.aoid.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.Vo.AoidYeildVo;
import com.gct.aoid.entity.Vo.CalRemakeVo;
import com.gct.aoid.entity.Vo.LiqWarringVo;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.TimingSummaryJob;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.common.core.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gxt
 * @since 2021-01-28
 */
@Api(value = "AoidDailyYieldController", description = "功图量液")
@RestController
@RequestMapping("/aoidDailyYield")
@Slf4j
public class AoidDailyYieldController {
    @Autowired
    private AoidDailyYieldService aoidDailyYieldService;
    @Autowired
    public LiqProdDailyCalService liqProdDailyCalService;

    @Value("${gct.quartz.jobs.cal-daily-liq-job.param.startTimeEveryDay}")
    private String startTimeEveryDay;
    @Autowired
    BaseDataClient baseDataClient;

    public static AtomicInteger atomicInteger = new AtomicInteger(0);
    private static ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
    @ApiOperation(value = "根据井号和日期时间段查询量液的每日基本信息" ,notes = "根据井号和日期时间段查询量液的每日基本信息")
    @PostMapping("/getAoidDailyYieldInfo")
    public Result getAoidDailyYieldInfoByWellIdAndPorDatePeriod(@RequestBody AoidYeildVo aoidYeildVo){
        List<AoidDailyYield> list =  aoidDailyYieldService.getAoidDailyYieldInfoList(aoidYeildVo.getWellId(), aoidYeildVo.getStartDate(),aoidYeildVo.getStartDate());
        return list==null||list.size()==0?Result.error("计产数据未找到"):Result.ok(list);
    }

    @ApiOperation(value = "根据井号和日期时间段查询量液的每日基本信息page" ,notes = "根据井号和日期时间段查询量液的每日基本信息")
    @PostMapping("/getAoidDailyYieldInfoPage")
    public Result getAoidDailyYieldInfoByWellIdAndPorDatePeriodPage(@RequestBody AoidYeildVo aoidYeildVo){
        Page<AoidDailyYield> page =   aoidDailyYieldService.getAoidDailyYieldInfoByWellIdAndPorDatePeriodByPage(aoidYeildVo.getOrg(),aoidYeildVo.getWellId(), aoidYeildVo.getStartDate(),aoidYeildVo.getEndDate(),aoidYeildVo.getSize(),aoidYeildVo.getPage());
        return page ==null|| page.getRecords().size()==0?Result.error("未找到"):Result.ok(page);
    }

    @ApiOperation(value = "根据井号和日期时间段查询量液的每日基本信息page" ,notes = "根据井号和日期时间段查询量液的每日基本信息")
    @PostMapping("/getAoidDailyYieldInfoPageNew")
    public Result getAoidDailyYieldInfoPage(@RequestBody AoidYeildVo aoidYeildVo){
        Page<AoidDailyYield> page =   aoidDailyYieldService.getAoidDailyYieldInfoPage(aoidYeildVo);
        return page ==null|| page.getRecords().size()==0?Result.error("未找到"):Result.ok(page);
    }


    @ApiOperation(value="get default data from aoid_daily_yield,may be return null")
    @GetMapping("/getDefaultData")
    public Result getDefaultData(){
        AoidDailyYield defaultData = aoidDailyYieldService.getDefaultData();
        return defaultData ==null?Result.error("Can not find available data for default view."):Result.ok(defaultData);
    }

    @ApiOperation(value="get default data from aoid_daily_yield,may be return null")
    @PostMapping("/getDefaultDataNew")
    public Result getDefaultDataNew(@RequestBody String wellIdSet){
        List<AoidDailyYield> defaultData = aoidDailyYieldService.getDefaultDataNew(wellIdSet);
        return defaultData.isEmpty()?Result.error("Can not find available data for default view."):Result.ok(defaultData);
    }

    @ApiOperation(value="get default data from aoid_daily_yield,may be return null")
    @PostMapping("/exportExcel")
    public Result exportExcel(@RequestBody JSONObject param){
        Integer bs =  param.getInteger("bs");
        String orgId = param.getString("orgId");
        String wellId = param.getString("wellId");
        String startDate = param.getString("startDate");
        String endDate = param.getString("endDate");
        String wellIdSet = param.getString("wellIdSet");
        JSONObject jo = aoidDailyYieldService.exportExcel(bs,orgId,wellId,startDate,endDate,wellIdSet);
        return jo.getJSONArray("data").size() == 0 ? Result.error("无数据") : Result.ok(jo);
    }


    @PostMapping("/getAllAoidInfoAuthorize")
    public Result getAllMeasureInfoAuthorize(@RequestBody JSONObject param) {
        Integer bs =  param.getInteger("bs");
        String orgId = param.getString("orgId");
        String wellId = param.getString("wellId");
        String startDate = param.getString("startDate");
        String endDate = param.getString("endDate");
        Integer page = param.getInteger("page");
        Integer size = param.getInteger("size");
        String wellIdSet = param.getString("wellIdSet");
        JSONObject jo = aoidDailyYieldService.getAllMeasureInfo(bs,orgId,wellId,startDate,endDate,page,size,wellIdSet);
        return jo.getJSONArray("data").size() == 0 ? Result.error("无数据") : Result.ok(jo);
    }

    @PostMapping("/CalculateProdDateData")
    @ApiOperation(value="重新计算指定时间的量液汇总数据,时间格式为 yyyy-MM-dd ")
    public void  CalculateProdDateData (@RequestBody CalRemakeVo calRemakeVo)  {


        if (atomicInteger.get()!=0) {
           // writeLock.unlock();
            return;
        }
        //String time ="00:00:00";
        String well = calRemakeVo.getWell();
        String[] orgIds = calRemakeVo.getOrgIds().toArray(new String[0]);
        ExecutorService pool = Executors.newSingleThreadExecutor();
        pool.execute(()->{
            atomicInteger.set(2);
            TimingSummaryJob.threadLocal.set(true);
            // liqProdDailyCalService.calculate(list,calRemakeVo.getProdDateStart() + time,calRemakeVo.getProdDateEnd() + time);
            log.error("reCal--->start");
            try {
                aoidDailyYieldService.calculateProdDateData(well,orgIds, calRemakeVo.getProdDateStart(), calRemakeVo.getProdDateEnd(), startTimeEveryDay);
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            }
            //liqProdDailyCalService.calculate(orgIds,calRemakeVo.getProdDateStart(),startTimeEveryDay);
            atomicInteger.set(1);
            log.error("reCal--->end");
            // 0 没有计算任务 1 计算完成  2 计算中
        });
    }

    @GetMapping("/getCalTaskExecutor")
    public Result getCalTaskExecutor (){
        int i = atomicInteger.get();
        if (i==1){
            atomicInteger.set(0);
        }
        return Result.ok(i);
    }

    @GetMapping("/getAoidLiqProdDailyForSevenDay")
    public Result getAoidLiqProdDailyForSevenDay(){
        List<Map<String, Object>> aoidLiqProdDailyForSevenDay = aoidDailyYieldService.getAoidLiqProdDailyForSevenDay();
        return Result.ok(aoidLiqProdDailyForSevenDay);
    }


    public Result getSingleHistoryYieldCurveOrChart(@RequestBody AoidYeildVo aoidYeildVo){
        List<AoidDailyYield> list =  aoidDailyYieldService.getAoidDailyYieldInfoList(aoidYeildVo.getWellId(), aoidYeildVo.getStartDate(),aoidYeildVo.getStartDate());
        return list==null||list.size()==0?Result.error("未找到"):Result.ok(list);
    }


    @ApiOperation(value = "getSingleHistoryYieldCurveOrChart(根据井号和时间区间获取单井历史产量曲线/柱状图)", notes = "根据井号和时间区间获取单井历史产量曲线/柱状图")
    @GetMapping("/getSingleHistoryYieldCurveOrChart")
    public Result getSingleHistoryYieldCurveOrChart(@RequestParam("wellName") @ApiParam("输入井号") String wellName,
                                                    @RequestParam("beginDate") @ApiParam("请输入开始时间") String beginDate,
                                                    @RequestParam("endDate") @ApiParam("请输入结束时间") String endDate,
                                                    @RequestParam("bs") @ApiParam("请输入标识") String bs) throws ParseException {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date = sdf.parse(beginDate);
        String beginDate2 = sdf.format(date);

        return aoidDailyYieldService.getSingleHistoryYieldCurveOrChart(wellName,beginDate2,endDate,bs);
    }



    @PostMapping("/getLiqWarring")
    public Result getLiqWarring(@RequestBody LiqWarringVo liqWarringVo){
        Map<String, Object> data =  aoidDailyYieldService.getLiqWarring(liqWarringVo);
        return Result.ok(data);
    }

    @PostMapping("/getCalErrorRateByDate")
    public Result getCalErrorRateByDate(@RequestBody JSONObject jsonObject) throws ParseException {
        return Result.ok(aoidDailyYieldService.getCalErrorRateByDate(jsonObject));
    }

    @PostMapping("/getCalErrorRateDiagram")
    public Result getCalErrorRateDiagram(@RequestBody JSONObject jsonObject){
        return Result.ok(aoidDailyYieldService.getCalErrorRateDiagram(jsonObject));
    }

    /**
     * 分页查询误差率详细井号
     * @param pageNum
     * @param pageSize
     * @param zyq
     * @param wcStart
     * @param wcEnd
     * @param cylStart
     * @param cylEnd
     * @param dateStart
     * @param dateEnd
     * @return
     */


    @ApiOperation(value = "分页查询", notes = "分页查询误差率详细井号")
    @GetMapping("/findWellWcByPages")
    public Result findWellWcByPages(
            @ApiParam(name = "pageNum", value = "当前页", required = true) @RequestParam("pageNum") Integer pageNum,
            @ApiParam(name = "pageSize", value = "页大小", required = true) @RequestParam("pageSize") Integer pageSize,
            @ApiParam(name = "zyq", value = "作业区") @RequestParam(value = "zyq", required = false) String zyq,
            @ApiParam(name = "wcStart", value = "误差开始") @RequestParam(value = "wcStart", required = false) String wcStart,
            @ApiParam(name = "wcEnd", value = "误差结束") @RequestParam(value = "wcEnd", required = false) String wcEnd,
            @ApiParam(name = "cylStart", value = "产液量开始") @RequestParam(value = "cylStart", required = false) String cylStart,
            @ApiParam(name = "cylEnd", value = "产液量结束") @RequestParam(value = "cylEnd", required = false) String cylEnd,
            @ApiParam(name = "jh", value = "井号") @RequestParam(value = "jh", required = false) String jh,
            @ApiParam(name = "dateStart", value = "日期开始") @RequestParam(value = "dateStart", required = false) String dateStart,
            @ApiParam(name = "dateEnd", value = "日期结束") @RequestParam(value = "dateEnd", required = false) String dateEnd
    ) {
        return Result.ok(aoidDailyYieldService.findWellWc(pageNum, pageSize, zyq, wcStart, wcEnd, cylStart,cylEnd,jh,dateStart,dateEnd));
    }



}

