package com.gct.aoid.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 
 * </p>
 *
 * @author gxt
 * @since 2021-12-21
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value="AoidLiquidAbnormal对象", description="液量异常信息表")
@TableName(schema = "aoid",value = "aoid_liquid_abnormal")
public class AoidLiquidAbnormal extends Model<AoidLiquidAbnormal> {


    @Component
    public static class DoubleSerialize extends JsonSerializer {
        @Override
        public void serialize(Object o, JsonGenerator jsonGenerator, SerializerProvider serializerProvider) throws IOException {
            if (o != null && o instanceof Double) {
                jsonGenerator.writeNumber(Math.round((Double) o * 100) / 100.0);
            }
        }
    }

    private static final long serialVersionUID=1L;

    @TableId(type = IdType.ASSIGN_UUID)
    private String id;

    @ApiModelProperty(value = "井号")
    private String wellId;

    @ApiModelProperty(value = "作业区")
    private String workArea;

    @ApiModelProperty(value = "井区")
    private String wellArea;

    @ApiModelProperty(value = "站点")
    private String station;

    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @ApiModelProperty(value = "生产时间")
    private LocalDateTime prodDate;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "油嘴")
    private Double oilMouth;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "泵径")
    private Double pumpDiameter;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "油压")
    private Double oilPressure;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "套压")
    private Double sheathPressure;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "回压")
    private Double backPressure;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "一个月内平均液量")
    private Double oneMonthAvgLiq;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "一个月内平均油量")
    private Double oneMonthAvgOil;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "一个月内平均含水率")
    private Double oneMonthAvgWaterCut;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "一个月内平均气量")
    private Double oneMonthAvgGas;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "十日内平均液量")
    private Double tenDayAvgLiq;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "十日内平均油量")
    private Double tenDayAvgOil;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "十日内平均含水率")
    private Double tenDayAvgWaterCut;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "十日内平均气量")
    private Double tenDayAvgGas;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "昨天液量")
    private Double yesterdayLiq;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "昨天油量")
    private Double yesterdayOil;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "昨天含水率")
    private Double yesterdayWaterCut;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "昨天气量")
    private Double yesterdayGas;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "今天液量")
    private Double todayLiq;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "今天油量")
    private Double todayOil;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "今天含水率")
    private Double todayWaterCut;

    @JsonSerialize(using = DoubleSerialize.class)
    @ApiModelProperty(value = "今天气量")
    private Double todayGas;

    @ApiModelProperty(value = "对比条件")
    private Integer contrastConditions;

    @ApiModelProperty(value = "波动比例")
    @JsonSerialize(using = DoubleSerialize.class)
    private Double fluctuationAmplitude;

    @ApiModelProperty(value = "描述")
    private String description;

    @ApiModelProperty(value = "异常信息")
    private String errorInfo;

    @Override
    protected Serializable pkVal() {
        return null;
    }

}
