package com.gct.aoid.feignclient;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.aoid.entity.WellAttr;
import com.gct.common.core.result.Result;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

/**
 * class name: BaseDataClient
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/1/25 下午6:02
 */
@FeignClient(value = "centralbase", fallback = BaseDataHystrix.class)
public interface    BaseDataClient {
    @RequestMapping(value = "/getLatestA2DailyData/{WellID}", method = RequestMethod.GET)
    String getLatestA2DailyData(@PathVariable("WellID") String wellID);

    @RequestMapping(value = "/WellSeriesInfo/WellSeriesInfo", method = RequestMethod.GET)
    String getWellSeriesInfo();

    @RequestMapping(value = "/cbPcProWellboreStatusDaily/getA2DataForCalProduction/{WellName}/{dateTime}", method = RequestMethod.GET)
    String getA2StatusDataForCalProduction(@PathVariable("WellName") String wellName, @PathVariable("dateTime") String dateTime);



    @RequestMapping(value = "/cbPcProWellboreVolDaily/getA2DataForCalVolDaily/{WellName}/{dateTime}", method = RequestMethod.GET)
    String getA2VolDataForCalProduction(@PathVariable("WellName") String wellName, @PathVariable("dateTime") String dateTime);

    @RequestMapping(value = "/cbPcDevWellAttr/selectOneByWellId", method = RequestMethod.GET)
    String selectOneByWellId(@RequestParam("well_id") String well_id);


    @RequestMapping(value = "/cbPcDevWellAttr/selectOneByWellIdList", method = RequestMethod.GET)
    String selectOneByWellIdList(@RequestParam("wellId") String wellId);




    @RequestMapping(value = "/cbPcDevWellAttr/selectOneByWellIdSeparator", method = RequestMethod.GET)
    String selectOneByWellIdSeparator( @RequestParam("wellId")  String wellId,
                                       @RequestParam("jobSystem")  String jobSystem);

    @RequestMapping(value = "/WellSource/jh", method = RequestMethod.GET)
    String getWellListByOrgID(@RequestParam("orgid") String org);

    @RequestMapping("/org/getOrgId/{orgId}")
    String getOrgId(@PathVariable("orgId") String orgId);


    @GetMapping("/org/getOrgByWellId/{wellId}")
    String getOrgByWellId(@PathVariable("wellId") String wellId);

    @RequestMapping(value = "/org/findAllIdByIdLevel", method = RequestMethod.GET)
    String findAllIdByIdLevel(@RequestParam("orgIdValue") String orgIdValue,@RequestParam("orgLevel")  int orgLevel);

    @RequestMapping(value = "/org/getOrgByOrgName", method = RequestMethod.GET)
    String getOrgByOrgName(@RequestParam("orgName") String orgName);



    @RequestMapping(value = "/cbPcDevWellAttr/selectOneByOrgId", method = RequestMethod.GET)
    String selectOneByOrgId(@RequestParam("OrgIds") List<String> OrgIds,
                            @RequestParam("wellId") String wellId,
                            @RequestParam("dateTime") String dateTime);


    @RequestMapping(value = "/cbPcDevWellAttr/getCbPcDevWellAttrMaxUpdateTime", method = RequestMethod.GET)
    String getCbPcDevWellAttrMaxUpdateTime(@RequestParam("wellId") String wellId,@RequestParam("realDate") String realDate,@RequestParam("flag") String flag);

    @RequestMapping(value = "/cbPcProWellboreVolDaily/getCbPcProWellboreVolDailyProdDate", method = RequestMethod.GET)
    String getCbPcProWellboreVolDailyProdDate(@RequestParam("wellId") String wellId,@RequestParam("identification") String identification,
                                              @RequestParam("ProdDate")  String ProdDate);



    @GetMapping("/org/getMaxLevelOrgId")
    String getMaxLevelOrgId();

    @GetMapping("/cbCdWellSource/getjhAll")
    String getjhAll();

    @GetMapping("/cbCdWellSource/getjhAllYieldError")
    String getjhAllYieldError();



    @GetMapping("/cbCdWellSource/getjhAllByorgid")
    String getjhAllByorgid(@RequestParam String orgid);

    @PostMapping("/cbPcDevWellAttr/addCbPcDevWellAttr")
    String addCbPcDevWellAttr(@RequestBody WellAttr wellattr);


    @PostMapping("/cbPcDevWellAttr/updateCbPcDevWellAttr")
    String updateCbPcDevWellAttr(@RequestBody WellAttr wellattr);

    @GetMapping("/cbPcDevWellAttr/updatePzxs")
    String updatePzxs(@RequestParam("well_id") String well_id,@RequestParam("pzxh") double pzxh);

    @GetMapping("/cbPcDevWellAttr/updateCoefficientByJh")
    String updateCoefficientByJh(@RequestParam("wellId") String wellId,@RequestParam("upateTime") String upateTime,@RequestParam("xs") double xs);


    @GetMapping("/cbPcDevWellAttr/updateis")
    String updateis(@RequestParam("well_id") String well_id,@RequestParam("sfpp") String sfpp,@RequestParam("dateTime") String dateTime,@RequestParam("fl") String fl);

    @GetMapping("/cbPcDevWellAttr/updateisempty")
    String updateisempty(@RequestParam("well_id") String well_id);


    @GetMapping("/org/getWellIdsByOrgIds")
    String getWellIdsByOrgIds(@RequestParam List<String> orgIds);

    @RequestMapping(value = "/CbPcProWellMeterTeam/getWellMeterTeamForCalProduction", method = RequestMethod.GET)
    String getWellMeterTeamForCalProduction(@RequestParam("WellName") String wellName, @RequestParam("dateTime") String dateTime);

    @PostMapping("/CbPcProWellMeterTeam/list")
    String getWellMeterTeamList(@RequestBody JSONObject jsonObject);

    @GetMapping("/org/getParAndSonOrg")
    String getParAndSonOrg(@RequestParam JSONArray orgStr);

    @GetMapping("/org/getOrgInfoByWellID")
    String getOrgInfoByWellID(@RequestParam String wellId);

    @GetMapping("/WellSource/jh")
    String getJh(@RequestParam("orgid") String orgid);

    @GetMapping ("/WellSource/getWellSourceByWellIdList")
    String getWellSourceByWellIdList(@RequestParam List<String> wellId);

    @GetMapping ("/WellSource/getWellSource")
    String getWellSource(@RequestParam String wellId);

    @GetMapping ("/WellSource/getWellSourceByStationName")
    String getWellSourceByStationName(@RequestParam String stationName,@RequestParam String producingAreaName);

    @RequestMapping(value = "/cbCdWellSource/getByWellId_Heavy", method = RequestMethod.GET)
    String getByWellId_Heavy(@RequestParam("wellName") String wellName);


    @GetMapping("/cbTempWellMechRuntime/getAllRuntimeData")
    String getAllRuntimeData();

    @PostMapping("/WellSource/getWellInfoByList")
    String getWellInfoByList(@RequestBody List<String> wellId);

    @GetMapping("/accessWell/getWellControlList")
    String getWellControlList();

    @GetMapping("/cbPcProWellboreVolDaily/getVolDailyListByProdDate")
    String getVolDailyListByProdDate(@RequestParam("prodDate") LocalDateTime prodDate ,@RequestParam("wellId") String  wellId);

    @GetMapping("/cbPcProWellboreVolDaily/hotWashCheck7day")
    String hotWashCheck7day(@RequestParam String wellId,@RequestParam String prodDate);

    @GetMapping("/org/org")
    String getOrganization(@RequestParam("parent_id") String parent_id);


}
