package com.gct.aoid.job;

import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.service.impl.CalibrationParameterService;
import com.gct.aoid.service.impl.YieldErrorAnalysisCalculationService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * class name: CalibrationParameterJob
 *
 * @author 王森玉
 * @version 1.0
 * @since 2022/6/20 下午14:48 周五
 */
@Slf4j
public class CalibrationParameterJob extends QuartzJobBean {

    public static ThreadLocal<Boolean> threadLocal = new ThreadLocal<Boolean>();

    @Autowired
    private CalibrationParameterService calibrationParameterService;

    @Autowired
    private BaseDataClient baseDataClient;
    /**
     * @Author 王森玉
     * @Description 每七天执行一次量液系数修正服务
     * @Date 14:23 2021.9.6
     * @Param
     * @return void
     **/
    protected void executeInternal(JobExecutionContext context){
        String jobName =  "CalibrationParameterJob";
        String jobGroup = "CalibrationParameterJob_Group";
        log.info("*===>[quartz-job:start]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        try {
             threadLocal.set(true);
            calibrationParameterService.Calibrationtask();

        } catch (Exception e) {
            log.error("[quartz-job:run error]JobName:{},JobGroup:{}\n-----------cause by:{}", jobName, jobGroup, e.getStackTrace());
            e.printStackTrace();
        }finally {
            threadLocal.remove();
        }
        //endregion
        log.info("*===>[quartz-job:completed]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
    }
}
