package com.gct.aoid.job;

import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.properties.JobParamConfigurationProperties;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.job.properties.ProdTimeCorrectProperty;
import com.gct.aoid.service.LiqProdDailyCalService;
import lombok.extern.slf4j.Slf4j;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;

/**
 * class name: TimingSummaryJob
 *
 * @author gxt
 * @version 1.0
 * @since 2021/8/20 上午11:20 周五
 */
@Slf4j
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class TimingSummaryJob extends QuartzJobBean {
    public static ThreadLocal<Boolean> threadLocal = new ThreadLocal<Boolean>();
    @Autowired
    public LiqProdDailyCalService service;

    @Autowired
    MutableJobConfigProperties config;

    @Autowired
    BaseDataClient baseDataClient;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        JobParamConfigurationProperties param = config.getCalDailyLiqJob().getParam();
        String jobName = "TimingSummaryJob";
        String jobGroup ="TIMINGSUMMARYJOB_GROUP";
        log.info("*===>[quartz-job:start]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
        try {
            LocalTime LocalTime = java.time.LocalTime.now();
            //LocalDate localDate = LocalDate.of(2021,07,20);;
            for (int i = 0; i < param.getProd().length; i++) {
                ProdTimeCorrectProperty prodTimeCorrectProperty = param.getProd()[i];
                if (prodTimeCorrectProperty.getDate().equals(LocalTime.format(DateTimeFormatter.ofPattern("HH:mm")))){
                    String[] strings = new String[]{prodTimeCorrectProperty.getOrgId()};
                    threadLocal.set(true);
                    service.calculate(strings, String.valueOf(LocalDate.now().minusDays(1)), param.getStartTimeEveryDay());
                }
            }
        } catch (Exception e) {
            log.error("[quartz-job:run error]JobName:{},JobGroup:{}\n-----------cause by:{}", jobName, jobGroup, e.getStackTrace());
            e.printStackTrace();
        }finally {
            threadLocal.remove();
        }
        //endregion
        log.info("*===>[quartz-job:completed]JobName:{},JobGroup:{},Time:{}", jobName, jobGroup, LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME));
    }
}
