package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidSinglegtYield;
import com.gct.aoid.entity.Vo.AoidYeildVo;
import com.gct.aoid.entity.Vo.LiqWarringVo;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.TimingSummaryJob;
import com.gct.aoid.mapper.AoidDailyYieldMapper;
import com.gct.aoid.mapper.AoidSinglegtYieldMapper;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jyx
 * @since 2021-01-28
 */
@Service
@Slf4j
public class AoidDailyYieldServiceImpl extends ServiceImpl<AoidDailyYieldMapper, AoidDailyYield> implements AoidDailyYieldService {
    private DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private DecimalFormat decimalFormat   =new DecimalFormat("#.00");
    @Autowired
    private AoidDailyYieldMapper aoidDailyYieldMapper;
    @Autowired
    BaseDataClient baseDataClient;
    @Autowired
    private AoidSinglegtYieldMapper aoidSinglegtYieldMapper;

    @Autowired
    private AoidSinglegtYieldService aoidSinglegtYieldService;

    @Value("${showNullLiq}")
    private boolean showNullLiq;
    @Value("${gct.quartz.jobs.cal-daily-liq-job.param.wellBatch}")
    private int wellBatch;

    @Autowired
    private LiqProdDailyCalService liqProdDailyCalService;
    @Override
    public Result getSingleHistoryYieldCurveOrChart(String wellName, String beginDate, String endDate,String bs) {
        QueryWrapper<AoidDailyYield> queryWrapper = new QueryWrapper<>();
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime begin = null;

        if ("1".equals(bs))
        {
            begin = LocalDateTime.parse((beginDate.length() == 10 ? beginDate + " 00:00:00" : beginDate), dtf).minusDays(6);
        }
        else if ("2".equals(bs)) {
            begin = LocalDateTime.parse((beginDate.length() == 10 ? beginDate + " 00:00:00" : beginDate), dtf);
        }
        LocalDateTime end = LocalDateTime.parse((endDate.length() == 10 ? endDate + " 00:00:00" : endDate), dtf);
        QueryWrapper<AoidDailyYield> wrapper = queryWrapper.select(
                        //日志
                        "a2_liq_prod_daily",
                        "a2_oil_prod_daily",
                        "a2_gas_prod_daily",
                        "a2_water_cut",
                        "test_liqu_vol_daily",
                        "yy",
                        "ty",
                        //计算
                        "liq_prod_daily",
                        //时间
                        "prod_date")
                .ge("prod_date", begin)
                .le("prod_date", end)
                .eq("well_id", wellName)
                .orderByAsc("prod_date");
        List<AoidDailyYield> aoidDailyYields = aoidDailyYieldMapper.selectList(wrapper);
        //封装结果集
        LinkedHashMap<String, Object> resultMap = new LinkedHashMap<>();
        //第一个eCharts图表
        LinkedHashMap<String, Object> eCharts1 = new LinkedHashMap<>();
        //横坐标
        List<String> xDate1 = new ArrayList<>();
        //系统液量
        LinkedHashMap<String, Object> eCharts1Liq = new LinkedHashMap<>();
        List<Double> eCharts1LiqData = new ArrayList<>();
        //A2日产液量
        LinkedHashMap<String, Object> a2eCharts1Liq = new LinkedHashMap<>();
        List<Double> a2eCharts1LiqData = new ArrayList<>();
        //计量间液量
        LinkedHashMap<String, Object> jljeCharts1Liq = new LinkedHashMap<>();
        List<Double> jljeCharts1LiqData = new ArrayList<>();
        //A2日产油量
        LinkedHashMap<String, Object> a2_oil_prod_daily = new LinkedHashMap<>();
        List<Double> a2_oil_prod_dailydata = new ArrayList<>();
        //A2日产气量
        LinkedHashMap<String, Object> a2_gas_prod_daily = new LinkedHashMap<>();
        List<Double> a2_gas_prod_dailydata = new ArrayList<>();
        //A2含水率
        LinkedHashMap<String, Object> a2_water_cut = new LinkedHashMap<>();
        List<Double> a2_water_cutdata = new ArrayList<>();
        //A2含水率
        LinkedHashMap<String, Object> yy = new LinkedHashMap<>();
        List<Double> yydata = new ArrayList<>();

        //A2含水率
        LinkedHashMap<String, Object> ty = new LinkedHashMap<>();
        List<Double> tydata = new ArrayList<>();
        //系统液量与A2液量的误差率
        LinkedHashMap<String, Object> error = new LinkedHashMap<>();
        List<Double> errordata = new ArrayList<>();




        if (aoidDailyYields!=null)
        {
            //遍历
            aoidDailyYields.forEach((AoidDailyYield) -> {
                double LiqProdDaily =0;
                double A2LiqProdDaily =0;
                double TestLiquVolDaily =0;
                double A2OilProdDaily =0;
                double A2GasProdDaily =0;
                double WaterCut =0;
                double Yy =0;
                double Ty =0;
                double wc =0;
                if(AoidDailyYield.getLiqProdDaily()==null || AoidDailyYield.getLiqProdDaily().equals(""))
                {
                    LiqProdDaily =0;
                }
                else {
                    LiqProdDaily = AoidDailyYield.getLiqProdDaily();
                }
                eCharts1LiqData.add(LiqProdDaily);//日产液量

                if(AoidDailyYield.getA2LiqProdDaily()==null || AoidDailyYield.getA2LiqProdDaily().equals(""))
                {
                    A2LiqProdDaily= 0;//A2液量
                }
                else {
                    A2LiqProdDaily = AoidDailyYield.getA2LiqProdDaily();//A2液量
                }
                a2eCharts1LiqData.add(A2LiqProdDaily);//A2液量

                if(AoidDailyYield.getTestLiquVolDaily()==null || AoidDailyYield.getTestLiquVolDaily().equals(""))
                {
                    TestLiquVolDaily=0;

                }
                else {
                    TestLiquVolDaily =AoidDailyYield.getTestLiquVolDaily();//计量间液量
                }
                jljeCharts1LiqData.add(TestLiquVolDaily);//计量间液量

                if(AoidDailyYield.getA2OilProdDaily()==null || AoidDailyYield.getA2OilProdDaily().equals(""))
                {
                    A2OilProdDaily=0;
                }
                else {
                    A2OilProdDaily = AoidDailyYield.getA2OilProdDaily();
                }
                a2_oil_prod_dailydata.add(A2OilProdDaily);//A2日产油量

                if(AoidDailyYield.getA2GasProdDaily()==null || AoidDailyYield.getA2GasProdDaily().equals(""))
                {
                    A2GasProdDaily=0;
                }
                else {
                    A2GasProdDaily = AoidDailyYield.getA2GasProdDaily();//A2日产气量
                }
                a2_gas_prod_dailydata.add(A2GasProdDaily);//A2日产气量

                if(AoidDailyYield.getWaterCut()==null || AoidDailyYield.getWaterCut().equals(""))
                {
                    WaterCut=0;
                }
                else {
                    WaterCut= AoidDailyYield.getWaterCut();//A2含水率
                }
                a2_water_cutdata.add(WaterCut);//A2含水率

                if(AoidDailyYield.getYy()==null || AoidDailyYield.getYy().equals(""))
                {
                    Yy=0;
                }
                else {
                    Yy= AoidDailyYield.getYy();//油压
                }
                yydata.add(Yy);//油压

                if(AoidDailyYield.getTy()==null || AoidDailyYield.getTy().equals(""))
                {
                    Ty=0;
                }
                else {
                    Ty= AoidDailyYield.getTy();//套压
                }
                tydata.add(Ty);//套压

                if(A2LiqProdDaily!=0.0)
                {
                    wc =round2(100*abs(A2LiqProdDaily,LiqProdDaily)/A2LiqProdDaily);
                }
                errordata.add(wc);
                xDate1.add(DateTimeFormatter.ofPattern("yyyy-MM-dd").format(AoidDailyYield.getProdDate()));
            });
        }

        //组装数据
        eCharts1Liq.put("name", "系统液量");
        eCharts1Liq.put("data", eCharts1LiqData);
        eCharts1Liq.put("unit", "t");
        a2eCharts1Liq.put("name", "A2日产液量");
        a2eCharts1Liq.put("data", a2eCharts1LiqData);
        a2eCharts1Liq.put("unit", "t");
        jljeCharts1Liq.put("name", "计量间液量");
        jljeCharts1Liq.put("data", jljeCharts1LiqData);
        jljeCharts1Liq.put("unit", "t");
        a2_oil_prod_daily.put("name", "A2日产油量");
        a2_oil_prod_daily.put("data", a2_oil_prod_dailydata);
        a2_oil_prod_daily.put("unit", "t");
        a2_gas_prod_daily.put("name", "A2日产气量");
        a2_gas_prod_daily.put("data", a2_gas_prod_dailydata);
        a2_gas_prod_daily.put("unit", "m³");
        a2_water_cut.put("name", "A2含水率");
        a2_water_cut.put("data", a2_water_cutdata);
        a2_water_cut.put("unit", "%");
        yy.put("name", "A2油压");
        yy.put("data", yydata);
        yy.put("unit", "t");
        ty.put("name", "A2套压");
        ty.put("data", tydata);
        ty.put("unit", "t");
        error.put("name", "误差率");
        error.put("data", errordata);
        error.put("unit", "%");
        eCharts1.put("xAxis", xDate1);
        eCharts1.put("series", new Object[]{eCharts1Liq, a2eCharts1Liq, jljeCharts1Liq,a2_oil_prod_daily,a2_gas_prod_daily,a2_water_cut,yy,ty,error});
        resultMap.put("eCharts1", eCharts1);
        return Result.ok(resultMap);
    }

    //计算绝对值
    public static double abs(double A2, double xtyl) {
        if(A2>xtyl){
            return A2-xtyl;
        }else{
            return xtyl-A2;
        }

    }
    //保留2位有效数字
    public static double round2(double in) {
        DecimalFormat    df   = new DecimalFormat("######0.00");
        return Double.valueOf(df.format(in));
    }


    @Override
    public List<AoidDailyYield> getAoidDailyYieldInfoList(String wellId, String startDate,String endDate) {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 00:00:00";
        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AoidDailyYield> queryWrapper = wrapper.lambda().eq(AoidDailyYield::getWellId, wellId).
                ge(AoidDailyYield::getProdDate, Timestamp.valueOf(startDate)).
                le(AoidDailyYield::getProdDate, Timestamp.valueOf(endDate)).orderByAsc(AoidDailyYield::getProdDate);
        List<AoidDailyYield> aoidDailyYieldList = aoidDailyYieldMapper.selectList(queryWrapper);
        return aoidDailyYieldList;
    }

    @Override
    public List<String> getWellIds(String org, String wellId) {
        if (Objects.nonNull(wellId) && !wellId.isEmpty()) return new ArrayList<>(Arrays.asList(new String[]{wellId}));

        if (Objects.isNull(org) || org.isEmpty()) return null;

        String resultStr = baseDataClient.getWellListByOrgID(org);
        if (Objects.isNull(resultStr) || resultStr.isEmpty()) return null;

        Result result = JSONObject.parseObject(resultStr, Result.class);
        if (Objects.isNull(result) || Objects.isNull(result.getData()))
            return null;
        JSONArray array = (JSONArray) result.getData();
        ArrayList<String> re = new ArrayList<>();
        for (int i = 0; i < array.size(); i++) {
            re.add(array.getJSONObject(i).getString("id"));
        }
        return re;
    }

    @Override
    public List<String> getWellIdsByNew(String org, String wellId,LocalDateTime startDate) {
        if (Objects.nonNull(wellId) && !wellId.isEmpty()) return new ArrayList<>(Arrays.asList(new String[]{wellId}));

        if (Objects.isNull(org) || org.isEmpty()) return null;

        String resultStr = baseDataClient.getWellListByOrgID(org);
        if (Objects.isNull(resultStr) || resultStr.isEmpty()) return null;
        //从机构信息表中获取所有的井号
        List<String> wellIdList = JSONObject.parseObject(resultStr).getJSONArray("data").stream().map(item -> ((JSONObject)item).get("id").toString()).collect(toList());

        List<AoidSinglegtYield> aoidSinglegtYields = aoidSinglegtYieldMapper.selectList(
                new QueryWrapper<AoidSinglegtYield>()
                        .select("distinct  well_id")
                        .ge("prod_date",startDate.minusDays(1))
                        .le("prod_date",startDate));
        List<String> singlegtList = new ArrayList();
        for (AoidSinglegtYield aoidSinglegtYield : aoidSinglegtYields) {
            singlegtList.add(aoidSinglegtYield.getWellId().toString());
        }
        List<String> Intersection = new ArrayList<>();
        //将两个井号取交集
        if (singlegtList.size() > 0 && wellIdList.size() > 0) {
            Intersection = singlegtList.stream().filter(wellIdList::contains).collect(toList());
        }
        return Intersection;
    }



    //@Override
    public Page<Map<String, Object>> getAoidDailyYieldInfoByWellIdAndPorDatePeriodByPage2(String org, String wellId, String startDate, int pageSize, int pageNum) {
        startDate = startDate + " 00:00:00";
        LocalDateTime localDateTime = LocalDateTime.parse(startDate, df);
        LocalDateTime localDateTime1 = localDateTime.plusDays(1);
        String end = localDateTime1.format(df);
        List<String> wellids = getWellIds(org, wellId);
        LambdaQueryWrapper<AoidDailyYield> queryWrapper = new LambdaQueryWrapper<>();
        if (wellids != null || wellids.size() > 0) {
            queryWrapper.in(AoidDailyYield::getWellId, wellids);
        }
        queryWrapper.ge(AoidDailyYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(startDate, df))).
                le(AoidDailyYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(end, df))).orderByAsc(AoidDailyYield::getProdDate);
        Page<Map<String, Object>> paramPage = new Page<>(pageNum, pageSize);
        Page<Map<String, Object>> mapPage = aoidDailyYieldMapper.selectMapsPage(paramPage, queryWrapper);
        for (Map<String, Object> map : mapPage.getRecords()) {
            String date = map.get("prod_date").toString().substring(0, 10);
            // String time = map.get("prod_date").toString().substring(11, 19);
//            map.put("prod_date",date+" "+time);
            map.put("prod_date", date);
        }
        return mapPage;
    }

    //@Override
    public Page<AoidDailyYield> getAoidDailyYieldInfoByWellIdAndPorDatePeriodByPage(String org, String wellId, String startDate,String endDate, int pageSize, int pageNum) {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 00:00:00";
        List<String> wellids = getWellIds(org, wellId);
        LambdaQueryWrapper<AoidDailyYield> queryWrapper = new LambdaQueryWrapper<>();
        if (wellids != null || wellids.size() > 0) {
            queryWrapper.in(AoidDailyYield::getWellId, wellids);
        }
        queryWrapper.ge(AoidDailyYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(startDate, df))).
                le(AoidDailyYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(endDate, df))).orderByAsc(AoidDailyYield::getProdDate);
        Page<AoidDailyYield> paramPage = new Page<>(pageNum, pageSize);
        return aoidDailyYieldMapper.selectPage(paramPage, queryWrapper);
    }


    @Override
    public AoidDailyYield getDefaultData() {
        AoidDailyYield aoidDailyYield = aoidDailyYieldMapper.selectOne(new LambdaQueryWrapper<AoidDailyYield>().orderByDesc(AoidDailyYield::getProdDate).last(" limit 1"));
        String orgByWellId = baseDataClient.getOrgByWellId(aoidDailyYield.getWellId());
        Result result = JSONObject.parseObject(orgByWellId, Result.class);
        if (orgByWellId!="") {
            String[] split = result.getData().toString().split("@");
            aoidDailyYield.setZyq(split[0]);
            aoidDailyYield.setJq(split[1]);
            aoidDailyYield.setZd(split[2]);
        }
        return aoidDailyYield;
    }

    @Override
    public List<AoidDailyYield>   getDefaultDataNew(String wellId) {
        Map map = JSONObject.parseObject(wellId, HashMap.class);
        List<String> wellIds = JSONObject.parseObject(map.get("wellIdSet").toString(), List.class);
        AoidDailyYield one = aoidDailyYieldMapper.selectOne(new QueryWrapper<AoidDailyYield>()
                .select("max(prod_date) prod_date"));
        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<AoidDailyYield>()
                .in("well_id",wellIds).eq("prod_date",one.getProdDate());
        List<AoidDailyYield> list = aoidDailyYieldMapper.selectList(wrapper);
        List<AoidDailyYield> collect = list.stream().filter(x -> Objects.isNull(x.getLiqProdDaily())).collect(toList());
        List<AoidDailyYield> temp = list.stream().filter(x -> !collect.contains(x)).collect(toList());
        temp = temp.stream().sorted(Comparator.comparingDouble(AoidDailyYield::getLiqProdDaily).reversed()).collect(toList());
        temp.addAll(collect);
        String wellInfoByWellId = baseDataClient.getWellInfoByList(wellIds);
        List list1 = JSONObject.parseObject(JSONObject.parseObject(wellInfoByWellId, Result.class).getData().toString(), ArrayList.class);
        for (Object o : list1) {
            for (AoidDailyYield dailyYield : temp) {
                if (((JSONObject) o).get("wellId").equals(dailyYield.getWellId())) {
                    String[] stationIds = ((JSONObject) o).getString("stationId").split("@");
                    dailyYield.setZyq(stationIds[0]);
                    dailyYield.setJq(stationIds[1]);
                    dailyYield.setZd(stationIds[2]);
                }
            }
        }
        return temp;
    }

    @Override
    public Page<AoidDailyYield> getAoidDailyYieldInfoPage(AoidYeildVo aoidYeildVo) {
        if (!Objects.isNull(aoidYeildVo)){
            List wellIdList = aoidYeildVo.getWellIdList();
            QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<AoidDailyYield>()
                    .in("well_id",wellIdList)
                    .ge("prod_date",Timestamp.valueOf(LocalDateTime.parse(aoidYeildVo.getStartDate() + " 00:00:00",df)))
                    .le("prod_date",Timestamp.valueOf(LocalDateTime.parse(aoidYeildVo.getEndDate() + " 00:00:00",df)))
                    .orderByAsc("prod_date");
            Page<AoidDailyYield> paramPage = new Page<>(aoidYeildVo.getPage(), aoidYeildVo.getSize());
            Page<AoidDailyYield> mapPage = aoidDailyYieldMapper.selectPage(paramPage, wrapper);
            return mapPage;
        }
        return null;
    }


    @Override
    public JSONObject getAllMeasureInfo(Integer bs, String orgId, String wellId, String startDate, String endDate, Integer curPage, Integer pageSize, String wellIdSet) {
        if (!StringUtils.isEmpty(endDate) && DateTimeUtil.formatDate(endDate).isAfter(LocalDate.now())){
            endDate = LocalDate.now() + "";
        }
        QueryWrapper<AoidDailyYield> wrapper = wrapper(bs, orgId, wellId, startDate, endDate, wellIdSet);

        Page<AoidDailyYield> page = new Page<>(curPage, pageSize);
        Page<AoidDailyYield> iPage = baseMapper.selectPage(page, wrapper);
        JSONObject jo = new JSONObject();
        jo.put("total", iPage.getTotal());
        jo.put("data", dataProcessing(iPage.getRecords()));
        return jo;
    }


    private QueryWrapper<AoidDailyYield> wrapper(Integer bs, String orgId, String wellId, String startDate, String endDate, String wellIdSet){
        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<AoidDailyYield>();
        wrapper= wrapper.ge("prod_date",DateTimeUtil.formatDateTime(startDate+" 00:00:00"))
                .le("prod_date",DateTimeUtil.formatDateTime(endDate+" 00:00:00"));

        if (bs!=2){
            if (showNullLiq){
                wrapper = wrapper.isNotNull("liq_prod_daily");
            }
            wrapper =  wrapper.orderByAsc("prod_date","liq_prod_daily");
        }
        else {
            wrapper =  wrapper.orderByAsc("prod_date").ge("errorgts",1);
        }
        if (StringUtils.isEmpty(wellId)) {
            if (StringUtils.isEmpty(orgId)){
                wrapper = wrapper.orderByAsc("prod_date");
            }
            if (!StringUtils.isEmpty(wellIdSet)){
                HashSet<String> set = JSONObject.parseObject(wellIdSet, HashSet.class);
                wrapper = wrapper.in("well_id", set).orderByAsc("prod_date");
            }
        } else {
            wrapper = wrapper.eq("well_id", wellId).orderByAsc("prod_date");
        }

        wrapper = wrapper.orderByDesc("well_id");
        return  wrapper;
    }

    private JSONArray dataProcessing(List<AoidDailyYield> aoidDailyYieldList ){
        JSONArray data = new JSONArray();
        AtomicInteger i = new AtomicInteger(1);
        List<String> wellIdList = new ArrayList<>();
        List<AoidDailyYield> collect = aoidDailyYieldList.stream().sorted(Comparator.comparing(AoidDailyYield::getProdDate)).collect(toList());
        collect.forEach(x -> {
            JSONObject rowObj = (JSONObject) JSONObject.toJSON(x);
            rowObj.put("id", i.getAndIncrement());
            if (x.getProdDate() != null) rowObj.put("endDate", LocalDateTime.parse(x.getProdDate().toString()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            if (rowObj.get("stroke")!=null && !rowObj.get("stroke").equals("")){
                BigDecimal bigDecimal  = BigDecimal.valueOf(Double.parseDouble(rowObj.get("stroke").toString()));
                rowObj.put("stroke",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (rowObj.get("frequence")!=null && !rowObj.get("frequence").equals("")) {
                BigDecimal bigDecimal  = BigDecimal.valueOf(Double.parseDouble(rowObj.get("frequence").toString()));
                rowObj.put("frequence", bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (rowObj.get("prodDate")!=null){
                rowObj.put("prodDate",rowObj.get("prodDate").toString().substring(0,10));
            }
            rowObj.put("prodDateYesterday",rowObj.getTimestamp("prodDate").toLocalDateTime().plusDays(-1));
            wellIdList.add(rowObj.getString("wellId"));
            data.add(rowObj);
        });

        String wellInfoByWellId = baseDataClient.getWellInfoByList(wellIdList);
        Result resultwellInfoByWellId = JSONObject.parseObject(wellInfoByWellId, Result.class);
        Object datawellInfoByWellId = resultwellInfoByWellId.getData();
        if (datawellInfoByWellId!=null)
        {
            List list = JSONObject.parseObject(JSONObject.parseObject(wellInfoByWellId, Result.class).getData().toString(), ArrayList.class);
            list.forEach(x->{
                data.forEach(m->{
                    if (((JSONObject)x).get("wellId").equals(((JSONObject)m).getString("wellId"))){
                        String[] stationIds = ((JSONObject) x).getString("stationId").split("@");
                        ((JSONObject)m).put("zyq",stationIds[0]);
                        ((JSONObject)m).put("jq",stationIds[1]);
                        ((JSONObject)m).put("zd",stationIds[2]);
                    }
                });
            });
        }

        return data;
    }
    @Override
    public JSONObject exportExcel(Integer bs, String orgId, String wellId, String startDate, String endDate, String wellIdSet) {
        QueryWrapper<AoidDailyYield> wrapper = wrapper(bs, orgId, wellId, startDate, endDate, wellIdSet);

        List<AoidDailyYield> aoidDailyYields = baseMapper.selectList(wrapper);
        JSONObject jo = new JSONObject();
        jo.put("data", dataProcessing(aoidDailyYields));
        return jo;
    }

    @Override
    public  List<Map<String, Object>> getAoidLiqProdDailyForSevenDay() {
        LocalDate localDate = LocalDate.now();
        String date = localDate.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        List<Map<String, Object>> aoidLiqProdDailyForSevenDay = baseMapper.getAoidLiqProdDailyForSevenDay(date);
        aoidLiqProdDailyForSevenDay.forEach(item->{
            item.put("proddate",item.get("proddate").toString().substring(0,item.get("proddate").toString().lastIndexOf(" ")));
        });
        return aoidLiqProdDailyForSevenDay;
    }

    @Override
    public Map<String, Object> getLiqWarring(LiqWarringVo liqWarringVo) {
        Map<String, Object> mapList = new HashMap<>();
        if (liqWarringVo.getWellId() != null && !liqWarringVo.getWellId().equals("") && liqWarringVo.getWellIdList() == null){
            List<String> list = new ArrayList<>();
            list.add(liqWarringVo.getWellId());
            liqWarringVo.setWellIdList(list);
        }else if (liqWarringVo.getOrgId() != null && !liqWarringVo.getOrgId().equals("")){
            String wellIdsByOrgIds = baseDataClient.getJh(liqWarringVo.getOrgId());
            List<Map<String, Object>> list = JSONObject.parseObject(JSONObject.parseObject(wellIdsByOrgIds, Result.class).getData().toString(), List.class);
            List<String> stringList = new ArrayList<>();
            list.forEach(x->stringList.add(x.get("id").toString()));
            liqWarringVo.setWellIdList(stringList);
        }
        Integer liqWarringCount = aoidDailyYieldMapper.getLiqWarringCount(LocalDateTime.parse(liqWarringVo.getPordDate() + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")), liqWarringVo.getWellIdList());
        Integer currentPage = Math.max((liqWarringVo.getCurrentPage() - 1) * liqWarringVo.getPageSize(), 0);
        List<Map<String, Object>> liqWarring = aoidDailyYieldMapper.getLiqWarring(LocalDateTime.parse(liqWarringVo.getPordDate() + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")), liqWarringVo.getWellIdList()
                ,currentPage,liqWarringVo.getPageSize());
        liqWarring.forEach(x->{
            x.put("prod_date",x.get("prod_date").toString().substring(0,x.get("prod_date").toString().lastIndexOf(" ")));
            if (x.get("stroke") != null){
                BigDecimal bigDecimal = new BigDecimal(x.get("stroke").toString());
                x.put("stroke",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("frequence") != null){
                BigDecimal bigDecimal1 = new BigDecimal(x.get("frequence").toString());
                x.put("frequence",bigDecimal1.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("liq_prod_dailyasyest") !=null){
                BigDecimal bigDecimal1 = new BigDecimal(x.get("liq_prod_dailyasyest").toString());
                x.put("liq_prod_dailyasyest",bigDecimal1.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("oil_prod_dailyasyest") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("oil_prod_dailyasyest").toString());
                x.put("oil_prod_dailyasyest",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("water_cutasyest") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("water_cutasyest").toString());
                x.put("water_cutasyest",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("liq_prod_dailyasten") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("liq_prod_dailyasten").toString());
                x.put("liq_prod_dailyasten",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("oil_prod_dailyasten") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("oil_prod_dailyasten").toString());
                x.put("oil_prod_dailyasten",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("water_cutasten") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("water_cutasten").toString());
                x.put("water_cutasten",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("liq_prod_dailyasmonth") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("liq_prod_dailyasmonth").toString());
                x.put("liq_prod_dailyasmonth",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("oil_prod_dailyasmonth") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("oil_prod_dailyasmonth").toString());
                x.put("oil_prod_dailyasmonth",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
            if (x.get("water_cutasmonth") !=null){
                BigDecimal bigDecimal = new BigDecimal(x.get("water_cutasmonth").toString());
                x.put("water_cutasmonth",bigDecimal.setScale(2, RoundingMode.HALF_UP));
            }
        });
        for (Map<String, Object> map : liqWarring) {
            String wellSource = baseDataClient.getOrgInfoByWellID(map.get("well_id").toString());
            HashMap hashMap = JSONObject.parseObject(JSONObject.parseObject(wellSource, Result.class).getData().toString(), HashMap.class);
            map.put("zyq", hashMap.get("gfatname"));
            map.put("jq", hashMap.get("fatname"));
            map.put("zd", hashMap.get("orgname"));
            map.put("zyqid", hashMap.get("gfat"));
            map.put("jqid", hashMap.get("fat"));
            map.put("zdid", hashMap.get("orgid"));
        }
        mapList.put("total",liqWarringCount);
        mapList.put("currentPage",currentPage);
        mapList.put("pageSize",liqWarringVo.getPageSize());
        mapList.put("data",liqWarring);
        return mapList;
    }

    @Override
    public Object getCalErrorRateByDate(JSONObject jsonObject) throws ParseException {

        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<>();
        wrapper.eq("prod_date",Timestamp.valueOf(jsonObject.getString("date")+" 00:00:00"))
                .in("well_id",jsonObject.getJSONArray("wellIdList"))
                .select("well_id","liq_prod_daily")
                .isNotNull("liq_prod_daily");

       List<AoidDailyYield> aoidDailyYieldList = baseMapper.selectList(wrapper);

        if (aoidDailyYieldList!=null)
        {
            for (int i=0;i<aoidDailyYieldList.size();i++)
            {
                String newDate = jsonObject.getString("date");
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
                Date time = simpleDateFormat.parse(newDate);

                Calendar calendar = Calendar.getInstance();
                calendar.setTime(time);

                int year = calendar.get(Calendar.YEAR);
                int month = calendar.get(Calendar.MONTH) + 1;
                int day = calendar.get(Calendar.DAY_OF_MONTH);

                String monthValue = month >= 10 ? String.valueOf(month) : "0" + month;
                String dayValue = day >= 10 ? String.valueOf(day) : "0" + day;


                String dateEnd = String.valueOf(year)+'-'+monthValue+'-'+dayValue;



                String newDateValue = dateEnd+" 00:00:00";
                String dateString = LocalDateTime.parse(newDateValue,df).plusDays(1).format(df);
                List<AoidDailyYield> aoidDailyYieldValue = findByJhAoidDailyA2(aoidDailyYieldList.get(i).getWellId(),dateString);
                if (aoidDailyYieldValue.size()>0)
                {
                    aoidDailyYieldList.get(i).setA2LiqProdDaily(aoidDailyYieldValue.get(0).getA2LiqProdDaily());
                }

            }
        }

        return aoidDailyYieldList;


    }






    @Override
    public Object getCalErrorRateDiagram(JSONObject jsonObject) {
        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<>();
        wrapper.ge("prod_date",Timestamp.valueOf(jsonObject.getString("startDate")+" 00:00:00"))
                .le("prod_date",Timestamp.valueOf(jsonObject.getString("endDate")+" 23:59:59"))
                .in("well_id",jsonObject.getJSONArray("wellIdList"))
                .select("well_id","prod_date","liq_prod_daily","a2_liq_prod_daily")
                .isNotNull("liq_prod_daily").isNotNull("a2_liq_prod_daily");
        return baseMapper.selectList(wrapper);
    }





    @Override
    public List<AoidDailyYield> getCalErrorRateByDateToday(String date) {
        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<>();
        wrapper.eq("prod_date",Timestamp.valueOf(date+" 00:00:00"))
                .select("well_id","liq_prod_daily","a2_liq_prod_daily")
                .isNotNull("liq_prod_daily").isNotNull("a2_liq_prod_daily");
        return baseMapper.selectList(wrapper);
    }

    @Override
    public List<Map<String, Object>> getLiqDailyYestAndTenAndMonth(LocalDateTime prodDate) {
        return aoidDailyYieldMapper.getLiqDailyYestAndTenAndMonth(prodDate);
    }

    @Override
    public List<AoidDailyYield> findByJhAoidDaily(String jh, String start, String end)
    {
        return aoidDailyYieldMapper.findByJhAoidDaily(jh,start,end);
    }

    @Override
    public IPage<AoidDailyYield> findWellWc(Integer pageNum, Integer pageSize, String zyq, String wcStart,String wcEnd, String cylStart, String cylEnd,String jh,String  dateStart,String dateEnd) {

        Page<AoidDailyYield> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        Double wcStartFormat = new Double(wcStart);
        Double wcEndFormat = new Double(wcEnd);
        Double cylStartFormat = new Double(cylStart);
        Double cylEndFormat = new Double(cylEnd);

        IPage<AoidDailyYield> page = null;
        if (jh == null)
        {
            page=  aoidDailyYieldMapper.findWellWc(paramPage, zyq, wcStartFormat, wcEndFormat, cylStartFormat,cylEndFormat,dateStart,dateEnd);
        }
        else {
            page=  aoidDailyYieldMapper.findWellWcJhvalue(paramPage, wcStartFormat, wcEndFormat, cylStartFormat,cylEndFormat,jh,dateStart,dateEnd);
        }

        for (AoidDailyYield record : page.getRecords()) {

            DateTimeFormatter dfValue = DateTimeFormatter.ofPattern("yyyy-MM-dd");

            String dateStartYesterday = record.getProdDate().plusDays(-1).format(dfValue)+" 00:00:00";
            String dateEndYesterday = record.getProdDate().plusDays(-1).format(dfValue)+" 23:59:59";
            String dateStartYesterdayValue = record.getProdDate().plusDays(-1).format(dfValue);


            AoidDailyYield aoidDailyYieldValue = findWellWcJh(record.getWellId(),dateStartYesterday,dateEndYesterday);
            if (aoidDailyYieldValue!=null){
                Double LiqProdDailyYesterday = aoidDailyYieldValue.getLiqProdDaily();
                Double A2LiqProdDailyYesterday =aoidDailyYieldValue.getA2LiqProdDaily();
                Double A2YyYesterday =aoidDailyYieldValue.getYy();
                Double A2TyYesterday =aoidDailyYieldValue.getTy();
                String A2RzbzYesterday =aoidDailyYieldValue.getRzbz();
                Double wcYesterday =aoidDailyYieldValue.getWc();

                record.setYesterday(dateStartYesterdayValue);
                record.setLiqProdDailyYesterday(LiqProdDailyYesterday);
                if (record.getLiqProdDaily()>LiqProdDailyYesterday)
                {
                    record.setLiqProdDailyQs("上升");
                }
                else if (record.getLiqProdDaily()<LiqProdDailyYesterday) {
                    record.setLiqProdDailyQs("下降");
                }
                else {
                    record.setLiqProdDailyQs("相等");
                }


                record.setYyYesterday(A2YyYesterday);
                if (record.getYy()>A2YyYesterday)
                {
                    record.setYyQs("上升");
                }
                else if (record.getYy()<A2YyYesterday) {
                    record.setYyQs("下降");
                }
                else {
                    record.setYyQs("相等");
                }

                record.setTyYesterday(A2TyYesterday);
                if (record.getTy()>A2TyYesterday)
                {
                    record.setTyQs("上升");
                }
                else if (record.getTy()<A2TyYesterday) {
                    record.setTyQs("下降");
                }
                else {
                    record.setTyQs("相等");
                }


                record.setA2LiqProdDailyYesterday(A2LiqProdDailyYesterday);
                if (record.getA2LiqProdDaily()>A2LiqProdDailyYesterday)
                {
                    record.setA2LiqProdDailyQs("上升");
                }
                else if (record.getA2LiqProdDaily()<A2LiqProdDailyYesterday) {
                    record.setA2LiqProdDailyQs("下降");
                }
                else {
                    record.setA2LiqProdDailyQs("相等");
                }

                record.setWcYesterday(round2(100*wcYesterday));
                if (record.getWc()>wcYesterday)
                {
                    record.setWcQs("上升");
                }
                else if (record.getWc()<wcYesterday) {
                    record.setWcQs("下降");
                }
                else {
                    record.setWcQs("相等");
                }

                record.setRzbzYesterday(A2RzbzYesterday);

            }
            record.setWc(round2(100*record.getWc()));

        }

        return page;
    }

    @Override
    public void calculateProdDateData(String well,String[] orgs, String startDate, String endDate, String startTimeEveryDay) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        LocalDateTime start = DateTimeUtil.formatDateTime(startDate + " " + startTimeEveryDay);
        LocalDateTime end = DateTimeUtil.formatDateTime(endDate+ " " + startTimeEveryDay);
        if (!Objects.equals(well, "")) {
            List<List<String>> wellIdList = new ArrayList<>();
            wellIdList.add(Collections.singletonList(well));
            for (List<String> strings : wellIdList) {
                calculate(strings, start, end);
            }
        }
        else {
            Set<String> distinctOrgIDs = new HashSet<>();
            for (String org : orgs)
                if (distinctOrgIDs.add(org)) {
                    List<String> wellIds = getWellIdsByNew(org, "", start);
                    List<List<String>> wellIdList = new ArrayList<>();
                    Stream.iterate(0, n -> n + 1).limit(wellIds.size()).forEach(item -> {
                        wellIdList.add(wellIds.stream().skip((long) item * wellBatch).limit(wellBatch).collect(Collectors.toList()));
                    });
                    for (List<String> strings : wellIdList) {
                        calculate(strings, start, end);
                    }
                }
        }

    }




    public void calculate(List<String> wellIds, LocalDateTime startDate, LocalDateTime endDate) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {

        for (; Duration.between(startDate, endDate).toDays() >= 0; startDate = startDate.plusDays(1)) {
            LocalDateTime start = startDate.minusDays(1);
            List<AoidSinglegtYield> lis = aoidSinglegtYieldService.getAoidSinglegtYieldInfoByWellIdAndPorDatePeriodNew(wellIds, start, startDate);
            if (lis != null && lis.size() > 0) {
                Collection<List<AoidSinglegtYield>> values = lis.stream().collect(Collectors.groupingBy(AoidSinglegtYield::getWellId)).values();
                for (List<AoidSinglegtYield> value : values) {
                    if (value != null && value.size() > 0) {
                        Double time = null;
                        if (TimingSummaryJob.threadLocal.get()) {
                            String a2VolDataForCalProduction = baseDataClient.getA2VolDataForCalProduction(value.get(0).getWellId(), DateTimeUtil.formatDateStr(startDate));//可能用start 或 start+1
                            Object data = JSONObject.parseObject(a2VolDataForCalProduction, Result.class).getData();
                            if (Objects.isNull(data)) return;
                            HashMap hashMap = JSONObject.parseObject(data.toString(), HashMap.class);
                            if (hashMap != null && hashMap.size() > 0) {
                                BigDecimal prodTime = (BigDecimal) hashMap.get("prodTime");
                                time = Double.valueOf(prodTime.toString());
                            }
                        }
                        liqProdDailyCalService.calculate(value, startDate, startDate.plusDays(1), time);
                    }
                }
            }
        }
    }


    /**
     * 分页查询误差率详细井号(根据井号查询)
     * @param jh
     * @param dateStart
     * @param dateEnd
     * @return
     */
    public AoidDailyYield findWellWcJh( String jh,String  dateStart,String dateEnd) {
        return aoidDailyYieldMapper.findWellWcJh(jh,dateStart,dateEnd);
    }


    /**
     * 查询量液汇总数据（针对A2为0不作参考）
     * @param jh
     * @return
     */
    @Override
    public List<AoidDailyYield> findByJhAoidDailyA2(String jh,String prodDate) {

        return aoidDailyYieldMapper.findByJhAoidDailyA2(jh,prodDate);

    }

}
