package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidDeviceLiq;
import com.gct.aoid.entity.AoidDeviceWell;
import com.gct.aoid.entity.AoidProductRetrieval;
import com.gct.aoid.entity.Vo.ProductRetrievalVo;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.mapper.AoidProductRetrievalMapper;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidDeviceLiqService;
import com.gct.aoid.service.AoidDeviceWellService;
import com.gct.aoid.service.AoidProductRetrievalService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 产量检索表 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-10-24
 */
@Service
public class AoidProductRetrievalServiceImpl extends ServiceImpl<AoidProductRetrievalMapper, AoidProductRetrieval> implements AoidProductRetrievalService {

    @Autowired
    BaseDataClient baseDataClient;

    @Autowired
    AoidDeviceLiqService aoidDeviceLiqService;
    @Autowired
    AoidDeviceWellService aoidDeviceWellService;

    @Autowired
    AoidDailyYieldService aoidDailyYieldService;

    @Override
    public List<AoidProductRetrieval> getListData(ProductRetrievalVo productRetrievalVo) {
        LambdaQueryWrapper<AoidProductRetrieval> wrapper = new LambdaQueryWrapper<>();
        if (!StringUtil.isBlankOrEmpty(productRetrievalVo.getWellId())) {
            wrapper = wrapper.eq(AoidProductRetrieval::getWellId, productRetrievalVo.getWellId());
        } else {
            String orgId = null;
            if (!StringUtil.isBlankOrEmpty(productRetrievalVo.getStation())) {
                orgId = productRetrievalVo.getStation();
            } else if (!StringUtil.isBlankOrEmpty(productRetrievalVo.getWellArea())) {
                orgId = productRetrievalVo.getWellArea();
            } else if (!StringUtil.isBlankOrEmpty(productRetrievalVo.getWorkArea())) {
                orgId = productRetrievalVo.getWorkArea();
            }
            if (!StringUtil.isBlankOrEmpty(orgId)) {
                List<Map<String, String>> list = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getWellListByOrgID(orgId), Result.class).getData().toString(), List.class);
                List<String> wellIds = new ArrayList<>();
                list.forEach(x -> wellIds.add(x.get("id")));
                wrapper = wrapper.in(AoidProductRetrieval::getWellId, wellIds);
            }
        }
        if (productRetrievalVo.getProdDate().length == 2) {
            wrapper = wrapper.ge(AoidProductRetrieval::getProdDate, DateTimeUtil.formatDateTime(productRetrievalVo.getProdDate()[0] + " 00:00:00"))
                    .le(AoidProductRetrieval::getProdDate, DateTimeUtil.formatDateTime(productRetrievalVo.getProdDate()[1] + " 00:00:00"));
        }
        wrapper.orderByDesc(AoidProductRetrieval::getDetermineMode).orderByAsc(AoidProductRetrieval::getIsSummary);
        return baseMapper.selectList(wrapper);
    }

    @Override
    public boolean updateData(AoidProductRetrieval aoidProductRetrieval) {
        return updateById(aoidProductRetrieval);
    }

    @Override
    public void insertData(String date) {
        String time = " 00:00:00";
        List<AoidProductRetrieval> list = new ArrayList<>();
        List<AoidDeviceWell> deviceWells = aoidDeviceWellService.getBaseMapper().selectList(new LambdaQueryWrapper<AoidDeviceWell>()
                .orderByAsc(AoidDeviceWell::getDeviceName).orderByAsc(AoidDeviceWell::getIsSummary));
        if (!deviceWells.isEmpty()) {
            deviceWells.forEach(device -> {
                AoidProductRetrieval retrieval = new AoidProductRetrieval();
                retrieval.setWellId(device.getWellName())
                        .setAllowableError(device.getAllowableError())
                        .setWellSystem(device.getWellSystem())
                        .setDownHoleStructure(device.getDownHoleStructure())
                        .setPumpType(device.getPumpType())
                        .setOilMachineType(device.getOilMachineType())
                        .setWorkArea(device.getWorkArea())
                        .setIsSummary(Integer.valueOf(device.getIsSummary()))
                        .setDetermineMode(device.getDeviceName())
                        .setProdDate(DateTimeUtil.formatDateTime(date + time));
                list.add(retrieval);
            });
        }
        for (AoidProductRetrieval retrieval : list) {
            List<AoidDeviceLiq> deviceLiqs = aoidDeviceLiqService.getBaseMapper().selectList(new LambdaQueryWrapper<AoidDeviceLiq>()
                    .eq(AoidDeviceLiq::getWellName, retrieval.getWellId())
                    .between(AoidDeviceLiq::getGetTime, DateTimeUtil.formatDateTime(date + time),
                            DateTimeUtil.formatDateTime(date + time).plusDays(1)));
            if (deviceLiqs.isEmpty()) continue;
            List<AoidDailyYield> dailyYields = aoidDailyYieldService.getBaseMapper().selectList(new LambdaQueryWrapper<AoidDailyYield>()
                    .eq(AoidDailyYield::getWellId, retrieval.getWellId()).eq(AoidDailyYield::getProdDate, DateTimeUtil.formatDateTime(date + time)));
            if (dailyYields.isEmpty()) {
                OptionalDouble average = deviceLiqs.stream().filter(x -> Objects.nonNull(x.getLiquid()) && Double.parseDouble(x.getLiquid()) > 0)
                        .mapToDouble(x -> Double.parseDouble(x.getLiquid())).average();
                if (average.isPresent()) {
                    double asDouble = average.getAsDouble();
                    retrieval.setDetermineVal(asDouble);
                }
            } else {
                Double liqProdDaily = dailyYields.get(0).getLiqProdDaily();
                retrieval.setSystemVal(liqProdDaily);
                double diffNum = Math.abs(Double.parseDouble(deviceLiqs.get(0).getLiquid()) - liqProdDaily);
                AoidDeviceLiq result = deviceLiqs.get(0);
                for (AoidDeviceLiq deviceLiq : deviceLiqs) {
                    double diffNumTemp = Math.abs(Double.parseDouble(deviceLiq.getLiquid()) - liqProdDaily);
                    if (diffNumTemp < diffNum) {
                        diffNum = diffNumTemp;
                        result = deviceLiq;
                    }
                }
                retrieval.setDetermineVal(Double.parseDouble(result.getLiquid()));
            }
        }
        Map<String, List<AoidProductRetrieval>> collect = list.stream().collect(Collectors.groupingBy(AoidProductRetrieval::getDetermineMode));
        for (Map.Entry<String, List<AoidProductRetrieval>> entry : collect.entrySet()) {
            Double temp = null;
            if (entry.getValue().isEmpty()) continue;
            for (AoidProductRetrieval retrieval : entry.getValue()) {
                if (retrieval.getIsSummary() == 1) {
                    if (Objects.nonNull(retrieval.getSystemVal())) {
                        if (Objects.isNull(temp)) temp = retrieval.getSystemVal();
                        else temp += retrieval.getSystemVal();
                    }
                }
            }
            for (AoidProductRetrieval retrieval : entry.getValue()) {
                retrieval.setSystemTotalVal(Objects.isNull(temp) ? retrieval.getSystemVal() : temp);
            }
            for (AoidProductRetrieval retrieval : entry.getValue()) {
                if (Objects.nonNull(retrieval.getDetermineVal()) && Objects.nonNull(retrieval.getSystemTotalVal())) {
                    BigDecimal bigDecimal = BigDecimal.valueOf(retrieval.getSystemTotalVal() - retrieval.getDetermineVal());
                    double doubleValue = bigDecimal.setScale(2, RoundingMode.HALF_UP).doubleValue();
                    retrieval.setCurrentError(doubleValue);
                } else if (Objects.nonNull(retrieval.getDetermineVal()) && Objects.isNull(retrieval.getSystemTotalVal())
                        && Objects.nonNull(retrieval.getSystemVal())) {
                    BigDecimal bigDecimal = BigDecimal.valueOf(retrieval.getSystemVal() - retrieval.getDetermineVal());
                    double doubleValue = bigDecimal.setScale(2, RoundingMode.HALF_UP).doubleValue();
                    retrieval.setCurrentError(doubleValue);
                }
            }
        }
        List<Map<String, Object>> access = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getWellControlList(), Result.class).getData().toString(), List.class);
        for (AoidProductRetrieval retrieval : list) {
            for (Map<String, Object> map : access) {
                if (retrieval.getWellId().equals(map.get("wellId"))) {
                    retrieval.setAccessMessage(Integer.parseInt(String.valueOf(map.get("accessStatus"))) == 1 ? "已接入" : "未接入");
                }
            }
        }
        List<AoidProductRetrieval> tempList = baseMapper.selectList(new LambdaQueryWrapper<AoidProductRetrieval>().eq(AoidProductRetrieval::getProdDate, DateTimeUtil.formatDateTime(date + time).minusDays(1)));
        if (!tempList.isEmpty()) {
            for (AoidProductRetrieval temp : tempList) {
                for (AoidProductRetrieval retrieval : list) {
                    if (retrieval.getWellId().equals(temp.getWellId())) {
                        retrieval.setAssessment(temp.getAssessment())
                                .setReason(temp.getReason());
                    }
                }
            }
        }
        list.forEach(x -> {
            LambdaQueryWrapper<AoidProductRetrieval> wrapper = new LambdaQueryWrapper<AoidProductRetrieval>()
                    .eq(AoidProductRetrieval::getWellId, x.getWellId()).eq(AoidProductRetrieval::getProdDate, x.getProdDate());
            List<AoidProductRetrieval> temp = baseMapper.selectList(wrapper);
            if (temp.isEmpty()) baseMapper.insert(x);
            else baseMapper.update(x, wrapper);
        });
    }
}
