package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.algorithm.CalDiagramProduction;
import com.gct.aoid.algorithm.CalPumpCard;
import com.gct.aoid.entity.AoidSinglegtYield;
import com.gct.aoid.entity.AoidSinglewellConfig;
import com.gct.aoid.entity.ProDataForCalProduction;
import com.gct.aoid.entity.WellAttr;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.properties.JobParamConfigurationProperties;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.mapper.AoidSinglegtYieldMapper;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.AoidSinglewellConfigService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.SGTUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.ParseException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author 王森玉
 * @since 2021-01-31
 */
@Service
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class AoidSinglegtYieldServiceImpl extends ServiceImpl<AoidSinglegtYieldMapper, AoidSinglegtYield> implements AoidSinglegtYieldService {



    @Autowired
    public AoidSinglegtYieldMapper aoidSinglegtYieldMapper;
    @Autowired
    private MutableJobConfigProperties properties;
    @Autowired
    BaseDataClient baseDataClient;


    @Autowired
    private AoidSinglewellConfigService aoidSinglewellConfigService;
    @Autowired
    private CalPumpCard calPumpCard;
    @Autowired
    private CalDiagramProduction calDiagramProduction;

    private final DateTimeFormatter dfValue = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    private DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    /**
     * 量液--功图平铺
     *
     * @param pageNum
     * @param pageSize
     * @param jh
     * @return
     * @author:王森玉
     */

    @Override
    public Result findAoidSinglegtYieldByPages(Integer pageNum, Integer pageSize, String jh) {

        //分页插件
        Page<Map<String, Object>> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.asc("prod_date"));
        //查询条件
        QueryWrapper<AoidSinglegtYield> queryWrapper = new QueryWrapper<>();
        QueryWrapper<AoidSinglegtYield> wapper = null;

        wapper = queryWrapper.select("well_common_name", "zdjg", "prod_date", "liq_prod_daily", "sgtFormatted", "oil_prod_daily", "stroke", "frequence", "bx", "hsl")
                .eq("well_common_name", jh);

        Page<Map<String, Object>> page = aoidSinglegtYieldMapper.selectMapsPage(paramPage, wapper);

        //格式化功图数据
        List<Map<String, Object>> records = page.getRecords();
        records.forEach((map) -> {
            map.put("sgt", SGTUtil.encodeToView((String) map.get("sgtFormatted")));
            map.put("prod_date", df.format(((Timestamp) map.get("prod_date")).toLocalDateTime()));
        });
        return Result.ok(page);
    }


    @Override
    public List<AoidSinglegtYield> findByJh(String jh) {

        return aoidSinglegtYieldMapper.findByJh(jh);

    }

    /**
     * 实时查询前一天单张功图液量信息数据
     * @param jh
     * @return
     */
    @Override
    public List<AoidSinglegtYield> findByJhbase(String jh,String start,String end) {

        return aoidSinglegtYieldMapper.findByJhbase(jh,start,end);

    }

    /**
     * 实时查询前一天单张功图液量信息数据(用于量液)
     * @param jh
     * @return
     */
    @Override
    public List<AoidSinglegtYield> findByJhbaseLy(String jh,String start,String end) {

        return aoidSinglegtYieldMapper.findByJhbaseLy(jh,start,end);

    }


    /**
     * 实时查询前一天单张功图液量信息数据（用于量液分析--诊断结果）
     * @param jh
     * @return
     */
    @Override
    public List<AoidSinglegtYield> findByJhbaseLyZdjg(String jh,String start,String end)
    {
        return aoidSinglegtYieldMapper.findByJhbaseLyZdjg(jh,start,end);
    }



    /**
     * 实时查询前一天单张功图液量信息数据（指定参数运行）
     * @param jh
     * @return
     */
    @Override
    public List<AoidSinglegtYield> findByJhbasecs(String jh,String start,String end) {

        return aoidSinglegtYieldMapper.findByJhbasecs(jh,start,end);

    }


    @Override
    public List<Map<String, Object>> getAoidSinglegtYieldListByOneDay(String wellId, String date) {
        JobParamConfigurationProperties param = properties.getCalDailyLiqJob().getParam();
        LocalDateTime start = DateTimeUtil.formatDateTime(date + " " + param.getStartTimeEveryDay()).minusDays(1);
        LocalDateTime end = DateTimeUtil.formatDateTime(date + " " + param.getStartTimeEveryDay());
        LambdaQueryWrapper<AoidSinglegtYield> queryWrapper = new LambdaQueryWrapper<AoidSinglegtYield>()
                .eq(AoidSinglegtYield::getWellId, wellId)
                .ge(AoidSinglegtYield::getProdDate, start)
                .lt(AoidSinglegtYield::getProdDate, end)
                .orderByAsc(AoidSinglegtYield::getProdDate);
        List<Map<String,Object>> lis = aoidSinglegtYieldMapper.selectMaps(queryWrapper);
        String orgByWellId = baseDataClient.getOrgInfoByWellID(wellId);
        Result result = JSONObject.parseObject(orgByWellId, Result.class);
        HashMap hashMap = JSONObject.parseObject(result.getData().toString(), HashMap.class);
        for (Map<String, Object> map : lis) {
            String date2 = map.get("prod_date").toString().substring(0, 10);
            String time = map.get("prod_date").toString().substring(11, 19);
            List<List<Double>> bgtList = null;
            if (map.get("bgt") != null)
                bgtList = SGTUtil.encodeToUseList((String) map.get("bgt"));
            List<List<Double>> sgtList = null;
            if (map.get("sgt") != null) sgtList = SGTUtil.encodeToUseList((String) map.get("sgt"));
            if (sgtList != null && sgtList.size() > 0)
                sgtList.add(sgtList.get(0));
            map.put("bgt", bgtList);
            map.put("sgt", sgtList);
            map.put("prod_date", date2 + " " + time);
            map.put("zyq", hashMap.get("gfat"));
            map.put("jq",hashMap.get("fat"));
            map.put("zd",hashMap.get("orgid"));
        }

        return lis;
    }

    @Override
    public List<Map<String, Object>> getAoidSinglegtYieldInfoByWellIdAndPorDatePeriod(String wellId, String startDate) {
        startDate = startDate + " 00:00:00";
        LocalDateTime localDateTime = LocalDateTime.parse(startDate, df);
        LocalDateTime localDateTime1 = localDateTime.plusDays(1);
        String end = localDateTime1.format(df);
        QueryWrapper<AoidSinglegtYield> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AoidSinglegtYield> queryWrapper = wrapper.lambda().eq(AoidSinglegtYield::getWellId, wellId).
                ge(AoidSinglegtYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(startDate, df))).
                le(AoidSinglegtYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(end, df))).orderByAsc(AoidSinglegtYield::getProdDate);
        List<Map<String, Object>> maps = aoidSinglegtYieldMapper.selectMaps(queryWrapper);
        for (Map<String, Object> map : maps) {
            String date = map.get("prod_date").toString().substring(0, 10);
            String time = map.get("prod_date").toString().substring(11, 19);
//            System.out.println("map = " + map);
//            System.out.println("(String) map.get(\"bgt\") = " + (String) map.get("bgt"));
//            System.out.println("(String) map.get(\"sgtFormatted\") = " + (String) map.get("sgtFormatted"));
            List<List<Double>> bgtList = null;
            if (map.get("bgt") != null)
                bgtList = SGTUtil.encodeToUseList((String) map.get("bgt"));
            List<List<Double>> sgtList = null;
            if (map.get("sgt") != null) sgtList = SGTUtil.encodeToUseList((String) map.get("sgt"));
            List<List<Double>> tmp = new ArrayList<>();
//            sgtList.remove(sgtList.size() - 1);
            if (sgtList != null && sgtList.size() > 0)
                sgtList.add(sgtList.get(0));
            map.put("bgt", bgtList);
            map.put("sgt", sgtList);
            map.put("prod_date", date + " " + time);
        }
        return maps;
    }

    @Override
    public List<AoidSinglegtYield> getAoidSinglegtYieldInfoByWellIdAndPorDatePeriod(String wellId, String start, String end) {
        return getAoidSinglegtYieldInfoByWellIdAndPorDatePeriod(wellId, LocalDateTime.parse(start, df), LocalDateTime.parse(end, df));
    }

    @Override
    public List<AoidSinglegtYield> getAoidSinglegtYieldInfoByWellIdAndPorDatePeriod(String wellId, LocalDateTime start, LocalDateTime end) {

        LambdaQueryWrapper<AoidSinglegtYield> queryWrapper = new LambdaQueryWrapper<AoidSinglegtYield>().eq(AoidSinglegtYield::getWellId, wellId).
                ge(AoidSinglegtYield::getProdDate, Timestamp.valueOf(start)).
                le(AoidSinglegtYield::getProdDate, Timestamp.valueOf(end)).orderByAsc(AoidSinglegtYield::getProdDate);
        List<AoidSinglegtYield> maps = aoidSinglegtYieldMapper.selectList(queryWrapper);

        return maps;
    }
    @Override
    public List<AoidSinglegtYield> getAoidSinglegtYieldInfoByWellIdAndPorDatePeriodNew(List<String> wellIds, LocalDateTime start, LocalDateTime end) {
        if ( wellIds==null ||wellIds.size()==0) return null;
        QueryWrapper<AoidSinglegtYield> queryWrapper = new QueryWrapper<AoidSinglegtYield>()
                .in("well_id",wellIds)
                .ge("prod_date",Timestamp.valueOf(start))
                .le("prod_date",Timestamp.valueOf(end))
                .orderByAsc("prod_date");
        List<AoidSinglegtYield> maps = aoidSinglegtYieldMapper.selectList(queryWrapper);

        return maps;
    }

    @Override
    public Page<Map<String, Object>> getAoidSinglegtYieldInfoByWellIdAndPorDatePeriodAndPages(String wellId, String startDate, String endDate, int page, int size) throws ParseException {

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-M-d HH:mm:ss");
        QueryWrapper<AoidSinglegtYield> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AoidSinglegtYield> queryWrapper = wrapper.lambda().eq(AoidSinglegtYield::getWellId, wellId).
                ge(AoidSinglegtYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(startDate + " 16:00:00", df).minusDays(1))).
                le(AoidSinglegtYield::getProdDate, Timestamp.valueOf(LocalDateTime.parse(endDate + " 16:00:00", df))).orderByAsc(AoidSinglegtYield::getProdDate);
        Page<Map<String, Object>> mapPage = aoidSinglegtYieldMapper.selectMapsPage(new Page(page, size), queryWrapper);


        for (Map<String, Object> record : mapPage.getRecords()) {
            Timestamp date_time = (Timestamp) record.get("prod_date");
            if (Objects.nonNull(date_time)) {
                LocalDateTime dateTime = date_time.toLocalDateTime();
                record.put("prod_date", dateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            }

            Timestamp compare_date_time = (Timestamp) record.get("comparison_prod_date");
            if (Objects.nonNull(compare_date_time)) {
                LocalDateTime compareDateTime = compare_date_time.toLocalDateTime();
                record.put("comparison_prod_date", compareDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")).substring(0,10));
            }

            //region put bgt
            if (Objects.nonNull(record.get("bgt"))) {
                List<List<Double>> bgtList = SGTUtil.encodeToUseList((String) record.get("bgt"));
                record.put("bgt", bgtList);
            }
            //endregion
            List<List<Double>> sgtList = new ArrayList<>();
            if (Objects.nonNull(record.get("sgt"))) {
                sgtList = SGTUtil.encodeToUseList((String) record.get("sgt"));
                if (Objects.nonNull(sgtList) && sgtList.size() > 1) {
                    sgtList.remove(sgtList.size() - 1);
                    sgtList.add(sgtList.get(0));
                }
            }

            record.put("sgt", sgtList);

            BigDecimal yxcc = (BigDecimal) record.get("yxcc");
            List<List<Double>> sgtLeft = new ArrayList<>();
            List<List<Double>> sgtUp = new ArrayList<>();
            List<List<Double>> sgtDown = new ArrayList<>();
            if (Objects.nonNull(sgtList) && sgtList.size() > 0 && Objects.nonNull(yxcc)) {
                for (int i = sgtList.size() >> 1; i < sgtList.size(); i++) {
                    double s = (double) sgtList.get(i).toArray()[0];
                    if (yxcc.doubleValue() > s) {
                        sgtLeft.add(sgtList.get(i - 1));
                    }
                }
                sgtLeft.add(sgtList.get(sgtList.size() - 1));
            }
            if (Objects.nonNull(sgtList)) {
                if (sgtList.size() > 0) {
                    sgtUp = new ArrayList<>(sgtList.subList(0, sgtList.size() / 2));
                }
                if (sgtList.size() >= 2) {
                    sgtDown = new ArrayList<>(sgtList.subList((sgtList.size() / 2) - 1, sgtList.size()));
                }
            }
            record.put("sgtLeft", sgtLeft);
            record.put("sgtUp", sgtUp);
            record.put("sgtDown", sgtDown);
        }
        return mapPage;
    }

    @Override
    public List<AoidSinglegtYield> getAoidSinglegtYieldInfogk(String wellId) {

        QueryWrapper<AoidSinglegtYield> queryWrapper = new QueryWrapper<AoidSinglegtYield>()
                .select("distinct(eczdjg)")
                .in("well_id",wellId)
                .ge("prod_date",Timestamp.valueOf("2021-11-04 00:00:00"))
                .le("prod_date",Timestamp.valueOf("2021-11-05 00:00:00"));
        List<AoidSinglegtYield> maps = aoidSinglegtYieldMapper.selectList(queryWrapper);

        return maps;
    }

    @Override
    public List<AoidSinglegtYield> getAoidSinglegtYieldMaxProdate(String wellId) {

        QueryWrapper<AoidSinglegtYield> queryWrapper = new QueryWrapper<AoidSinglegtYield>()
                .select("max(prod_date) prod_date")
                .eq("well_id",wellId)
                .notIn("a2_oil_prod_daily",0)
                .or().isNotNull("a2_oil_prod_daily");
        List<AoidSinglegtYield> maps = aoidSinglegtYieldMapper.selectList(queryWrapper);

        return maps;
    }

    /**
     * 针对A2液量频繁变化查询最新膨胀系数更新时间
     * @param wellId
     * @return
     */
    @Override
    public AoidSinglegtYield getAoidSinglegtYieldMaxUpdateTime(String wellId)
    {
        QueryWrapper<AoidSinglegtYield> queryWrapper = new QueryWrapper<AoidSinglegtYield>()
                .select("max(update_time) update_time")
                .eq("well_id",wellId);
        AoidSinglegtYield maps = aoidSinglegtYieldMapper.selectOne(queryWrapper);

        return maps;
    }


    @Override
    public Page<AoidSinglegtYield> getResultIdByWellId(String wellId, String date, boolean isOnlyShowErrorGt,int pageSize,int pageNum) {
        JobParamConfigurationProperties param = properties.getCalDailyLiqJob().getParam();
        LocalDateTime start = DateTimeUtil.formatDateTime(date + " " + param.getStartTimeEveryDay()).minusDays(1);
        LocalDateTime end = DateTimeUtil.formatDateTime(date + " " + param.getStartTimeEveryDay());
        QueryWrapper<AoidSinglegtYield> wrapper = new QueryWrapper<AoidSinglegtYield>()
                .eq("well_id",wellId)
                .ge("prod_date", start)
                .lt("prod_date", end)
                .eq("sgt_is_legal",isOnlyShowErrorGt ? 1 : 0);
        if (isOnlyShowErrorGt) wrapper = wrapper.ge("zdjg::int",1).orderByAsc("prod_date");
        else wrapper = wrapper.eq("zdjg::int",-1).orderByAsc("prod_date");
        Page<AoidSinglegtYield> objectPage = new Page<>(pageNum, pageSize);
        Page<AoidSinglegtYield> aoidSinglegtYieldPage = baseMapper.selectPage(objectPage, wrapper);
        for (AoidSinglegtYield record : aoidSinglegtYieldPage.getRecords()) {
            record.setSgt(JSON.toJSONString(SGTUtil.encodeToDoubleArray(record.getSgt())));
        }
        return aoidSinglegtYieldPage;
    }


    /**
     * 根据参数条件进行功图浏览
     * @param beginTime
     * @param endTime
     * @param orgid
     * @param wellId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public IPage<AoidSinglegtYield> getCvrresultByTimeBrowSingle(String beginTime, String endTime, String orgid, String wellId,Integer sgtIsLegal, Integer pageNum, Integer pageSize)
    {



        String orgIdEndValue = "";
        if (orgid ==null  )
        {
            orgIdEndValue =orgid;
        }
        else if (orgid.length()>=1 && orgid.length()<5){
            orgIdEndValue = orgid;
        }
        else if (orgid.length()>5)
        {
            String orgIdArrary[] =orgid.split(",");
            orgIdEndValue = orgIdArrary[1];
        }


        IPage<AoidSinglegtYield> page = null;

        if (wellId == null || wellId.length() == 0)
        {
            page=  getCvrresultByTime_brow_org(beginTime,endTime,orgIdEndValue,wellId,pageNum,pageSize);
        }
        else
        {
            page=  getCvrresultByTime_brow_wellid(beginTime,endTime,wellId,sgtIsLegal,pageNum,pageSize);
        }
        return page;

    }


    /**
     * 根据参数条件进行功图浏览(组织)
     * @param beginTime
     * @param endTime
     * @param wellId
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<AoidSinglegtYield> getCvrresultByTime_brow_org(String beginTime, String endTime,String orgid,String wellId, Integer pageNum, Integer pageSize)
    {


        List<String> well_id1=new ArrayList<>();


        List<String> wellIdSet = new ArrayList<>();
        Result jh2 = JSONObject.parseObject(baseDataClient.getJh(orgid), Result.class);
        List<Map<String, Object>> list = JSONObject.parseObject(jh2.getData().toString(), List.class);
        for (Map<String, Object> stringObjectMap : list) {
            wellIdSet.add(stringObjectMap.get("id").toString());
        }
        well_id1=wellIdSet;


        Page<AoidSinglegtYield> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.desc("prod_date"));


        IPage<AoidSinglegtYield> page = aoidSinglegtYieldMapper.getCvrresultByTime_brow(paramPage,well_id1,beginTime,endTime);
        for (AoidSinglegtYield record : page.getRecords()) {
            List<List<Double>> sgtList = new ArrayList<>();
            if (Objects.nonNull(record.getSgt())) {
                sgtList = SGTUtil.encodeToUseList((String) record.getSgt());
                if (Objects.nonNull(sgtList) && sgtList.size() > 1) {
                    sgtList.remove(sgtList.size() - 1);
                    sgtList.add(sgtList.get(0));
                }
            }

            record.setSgt(sgtList.toString());
        }

        return page;
    }


    /**
     * 根据参数条件进行功图浏览(井号)
     * @param beginTime
     * @param endTime
     * @param wellId
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<AoidSinglegtYield> getCvrresultByTime_brow_wellid(String beginTime, String endTime,String wellId,Integer sgtIsLegal, Integer pageNum, Integer pageSize)
    {

        Page<AoidSinglegtYield> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.asc("prod_date"));
        IPage<AoidSinglegtYield> page = null;
        if (sgtIsLegal == null )
        {
            page=  aoidSinglegtYieldMapper.getCvrresultByTime_brow_wellid(paramPage,wellId,beginTime,endTime);
        }
        else if (sgtIsLegal == 0 || sgtIsLegal == 1)
        {
            page=  aoidSinglegtYieldMapper.getCvrresultByTime_brow_wellidSgtIsLegal(paramPage,wellId,beginTime,endTime,sgtIsLegal);
        }
        for (AoidSinglegtYield record : page.getRecords()) {
            List<List<Double>> sgtList = new ArrayList<>();
            if (Objects.nonNull(record.getSgt())) {
                sgtList = SGTUtil.encodeToUseList((String) record.getSgt());
                if (Objects.nonNull(sgtList) && sgtList.size() > 1) {
                    sgtList.remove(sgtList.size() - 1);
                    sgtList.add(sgtList.get(0));
                }
            }

            record.setSgt(sgtList.toString());
        }

        return page;
    }

    /**
     * 添加单井属性数据
     * @param wellAttrValue
     * @return
     * author 王森玉
     * time 2021-8-16
     */
    @Override
    public void addCbPcDevWellAttr(WellAttr wellAttrValue) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        WellAttr wellAttr = new WellAttr();
        Map<String, Object> processparamValue = new HashMap<>();
        processparamValue.put("explain", "null");
        processparamValue.put("date","null");
        processparamValue.put("count", "null");
        processparamValue.put("trend", "null");
        double coefficient =0.0;
        wellAttr.setWellId(wellAttrValue.getWellId());
        wellAttr.setOilProductsSorting("1");

        Double aoidDailyValue = getA2Basedata(wellAttrValue.getWellId(),LocalDateTime.parse(wellAttrValue.getUpdateTime(),df).format(df));
        String dateStartCoefficient = LocalDateTime.parse(wellAttrValue.getUpdateTime(),df).minusDays(2).format(dfValue)+" 16:00:00";
        String dateEndCoefficient = LocalDateTime.parse(wellAttrValue.getUpdateTime(),df).minusDays(1).format(dfValue)+" 16:00:00";
        List<AoidSinglegtYield> resultLiq = findByJhbasecs(wellAttrValue.getWellId(),dateStartCoefficient,dateEndCoefficient);
        if (resultLiq.size()>0)
        {
            coefficient =  coefficientValue(aoidDailyValue.doubleValue(),resultLiq);
        }
        else {
            coefficient = 1.0;
        }
        wellAttr.setCoefficient(coefficient);


        wellAttr.setIsmatching("是");
        wellAttr.setUpdateTime(wellAttrValue.getUpdateTime());
        wellAttr.setMarkedBy(wellAttrValue.getMarkedBy());
        wellAttr.setMarkedParmExplain(JSONObject.toJSONString(processparamValue));
        baseDataClient.addCbPcDevWellAttr(wellAttr);
    }




    //获取A2基础数据
    @Override
    public Double getA2Basedata(String wellId, String A2ProdDate)throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException
    {
        double LiqProdDaily =0.0;

        ProDataForCalProduction a2DataVolYesterday;

        String a2dataStrVolYesterdaycalibration = baseDataClient.getWellMeterTeamForCalProduction(wellId, A2ProdDate);
        Result calibrationListresult = JSONObject.parseObject(a2dataStrVolYesterdaycalibration, Result.class);
        Object calibrationObject = calibrationListresult.getData();

        if (calibrationObject!=null)
        {

            JSONObject jo = (JSONObject) JSON.toJSON(calibrationObject);
            double testLiquVolDaily = jo.getDoubleValue("testLiquVolDaily");

            LiqProdDaily = testLiquVolDaily;
        }
        else {
            String a2dataStrVolYesterday = baseDataClient.getA2VolDataForCalProduction(wellId, A2ProdDate);
            a2DataVolYesterday = (ProDataForCalProduction) JSONObject.parseObject(a2dataStrVolYesterday, Result.class).toFiguredOutObject(ProDataForCalProduction.class);

            if (a2DataVolYesterday!=null)
            {
                LiqProdDaily = (a2DataVolYesterday.getLiqProdDaily()==null)?0:a2DataVolYesterday.getLiqProdDaily();
            }
        }





        return LiqProdDaily;
    }


    /**
     * 计算系数
     * @param a2
     * @param result1
     * @return
     */
    public  double coefficientValue(double a2, List<AoidSinglegtYield> result1) {

        double u =0; double d =10;
        double pre_pzxs =10;
        while (d-u>1e-6){
            double xs = (u+d)/2;
            double rclyxzsum=0.0;
            double data[] = new double[result1.size()];
            List<Double> list = new ArrayList<>();
            for (int n =0; n< result1.size(); n++)
            {
                long st = System.currentTimeMillis();
                data[n] +=  calWrapper(result1.get(n).getWellId(),result1.get(n).getSgt(),
                        result1.get(n).getStroke(),result1.get(n).getSgtIsLegal(),result1.get(n).getFrequence(),
                        result1.get(n).getEczdjg(),result1.get(n).getZdjg(),result1.get(n).getBj(),result1.get(n).getBs(),
                        result1.get(n).getHsl(),result1.get(n).getSmd(),result1.get(n).getYmd(),result1.get(n).getQyb(),
                        result1.get(n).getA2Dym(),xs);

            }
            for (int f=0; f<data.length;f++)
            {
                list.add(data[f]);
                List<Double> filters = new ArrayList<>();
                filters.add(0.0);
                list.removeAll(filters);

            }

            for (int p=0;p<list.size();p++)
            {
                rclyxzsum+=list.get(p);
            }
            double production = rclyxzsum/list.size();
            if(production-a2>a2*0.1)d=xs;
            else if(a2-production>a2*0.1)
            {
                u =xs;
            }
            else
            {
                if(Math.abs((u+d)/2)<0.1 || Math.abs((u+d)/2)>10 ||Math.abs(pre_pzxs-(u+d)/2)<=0.1)
                {
                    break;
                }
                if(a2-production>a2*0.1){
                    u=xs;
                }else if (production-a2>a2*0.1){
                    d=xs;
                }
            }
            pre_pzxs =(u+d)/2;
        }
        double xs2 = (u+d)/2;
        return  xs2;


    }


    /**
     * 量液入口
     * @param wellid
     * @param sgt
     * @param stroke
     * @param sgtIsLegal
     * @param frequence
     * @param eczdjg
     * @param zdjg
     * @param bj
     * @param bs
     * @param hsl
     * @param smd
     * @param ymd
     * @param qyb
     * @param dym
     * @param pzxs
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     */

    public double calWrapper(String wellid,String sgt, double stroke, int sgtIsLegal,double frequence, String eczdjg,String zdjg,double
            bj,double bs,double hsl,double smd,double ymd,double qyb,double dym,double pzxs)   {
        double rcyl1 = 0;
        rcyl1 = cal(sgt,stroke,sgtIsLegal,frequence,eczdjg,zdjg,bj,bs,hsl,smd,ymd,qyb,dym,pzxs);
        return   rcyl1;

    }

    /**
     * 稀油量液
     * @param sgt1
     * @param stroke
     * @param sgtIsLegal
     * @param frequence
     * @param eczdjg
     * @param zdjg
     * @param bj
     * @param bs
     * @param hsl
     * @param smd
     * @param ymd
     * @param qyb
     * @param dym
     * @param pzxs
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    public double cal(String sgt1, double stroke, int sgtIsLegal,double frequence, String eczdjg,String zdjg,double
            bj,double bs,double hsl,double smd,double ymd,double qyb,double dym,double pzxs) {


        double rcyl1 =0;
        AoidSinglewellConfig effectStrockModel = aoidSinglewellConfigService.getConfigOneObjByID(1);

        Double[][] bgt = null;
        Double[][] sgt = sgt1 == null ? null : SGTUtil.encodeToDoubleArray(sgt1);

        if(!"-1".equals(zdjg))
        {
            bgt = calPumpCard.cal_bgt(sgt, stroke, sgtIsLegal, 1.0, eczdjg);

            if (bgt!=null)
            {
                CalDiagramProduction.CalProductionResult production = new CalDiagramProduction.CalProductionResult();
                production = calDiagramProduction.calProduction(
                        sgt, bgt, stroke, frequence, zdjg, bj, bs, hsl, smd, ymd, qyb, dym, pzxs, effectStrockModel
                );
                rcyl1= production.rcyl1;
            }
        }
        return  rcyl1;
        //endregion
    }
    @Override
    public Page<AoidSinglegtYield> getCvrresultByTime(String beginTime, String endTime, String wellId, Integer pageNum, Integer pageSize) {
        LocalDateTime startDate = DateTimeUtil.formatDateTime(beginTime).minusDays(1);
        LocalDateTime endDate = DateTimeUtil.formatDateTime(endTime);
        Page<AoidSinglegtYield> page = baseMapper.selectPage(new Page<>(pageNum, pageSize), new LambdaQueryWrapper<AoidSinglegtYield>()
                .eq(AoidSinglegtYield::getWellId, wellId)
                .between(AoidSinglegtYield::getProdDate, startDate, endDate)
                .orderByAsc(AoidSinglegtYield::getProdDate));
        page.getRecords().forEach(x->{
            List<List<Double>> sgtList = new ArrayList<>();
            if (Objects.nonNull(x.getSgt())) {
                sgtList = SGTUtil.encodeToUseList(x.getSgt());
                if (Objects.nonNull(sgtList) && sgtList.size() > 1) {
                    sgtList.remove(sgtList.size() - 1);
                    sgtList.add(sgtList.get(0));
                }
            }
            x.setSgtUp(sgtList);
        });

        return page.getSize() == 0 ? null : page;
    }

    @Override
    public Object getOverlayPaging(Integer pageNum, Integer pageSize, String wellId, String beginTime, String endTime) {
        LocalDateTime startDate = DateTimeUtil.formatDateTime(beginTime).minusDays(1);
        LocalDateTime endDate = DateTimeUtil.formatDateTime(endTime);
        List<AoidSinglegtYield> list = baseMapper.selectList(new LambdaQueryWrapper<AoidSinglegtYield>()
                .eq(AoidSinglegtYield::getWellId, wellId).ge(AoidSinglegtYield::getProdDate, startDate).le(AoidSinglegtYield::getProdDate, endDate));
        long days = Duration.between(startDate, endDate).toDays();
        List<Map<String, Object>> resList = new ArrayList<>();
        for (long i = 0; i < days; i++) {
            Map<String, Object> resMap = new HashMap<>();
            List<Double[][]> doubles = new ArrayList<>();
            long finalI = i;
            list.forEach(x->{
                if (x.getProdDate().isBefore(startDate.plusDays(finalI +1)) && x.getProdDate().isAfter(startDate.plusDays(finalI))){
                    doubles.add(SGTUtil.encodeToDoubleArray(x.getSgt()));
                }
            });
            resMap.putIfAbsent(startDate.plusDays(i+1).toLocalDate().toString(),doubles);
            resList.add(resMap);
        }
        Map<String,Object> data = new HashMap<>();
        data.put("data",resList.stream().skip((long) (pageNum - 1) *pageSize).limit(pageSize).collect(Collectors.toList()));
        data.put("total",resList.size());
        return data;
    }
}
