package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidSinglegtYield;
import com.gct.aoid.entity.ProDataForCalProduction;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.TimingSummaryJob;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * class name: LiqProdDailyCalServiceImpl
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/7/16 上午10:08 Fri
 */
@Service
@Slf4j
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class LiqProdDailyCalServiceImpl implements LiqProdDailyCalService {
    @Autowired
    BaseDataClient baseDataClient;
    @Autowired
    AoidDailyYieldService aoidDailyYieldService;
    @Autowired
    AoidSinglegtYieldService aoidSinglegtYieldService;
    @Autowired
    MutableJobConfigProperties config;

    @Value("${aoid.cal-ratio}")
    private double calRatio;
    private final DateTimeFormatter formatterDate = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    private final DateTimeFormatter formatterTime = DateTimeFormatter.ofPattern("HH:mm:ss");
    private final DateTimeFormatter formatterDateTime = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private final String time0 = "00:00:00";
    @Value("${aoid.errorZDJG}")
    private String[] errorZDJG;

    @Override
    public void calculate(String[] orgIDs, String startDate, String startTimeEveryDay) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        log.info("input startDate:{}", startDate);
        calculate(orgIDs, LocalDateTime.parse(startDate + " " + time0, formatterDateTime), LocalTime.parse(startTimeEveryDay, formatterTime));


    }

    @Override
    public void calculate(String[] orgIDs, LocalDateTime startDate, LocalTime startTimeEveryDay) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        int wellBatch = config.getCalDailyLiqJob().getParam().getWellBatch();
        LocalDateTime localDateTime = DateTimeUtil.mergeToDateTime(startDate.toLocalDate(), startTimeEveryDay);
        Set<String> distinctOrgIDs = new HashSet<>();
        for (int i = 0; i < orgIDs.length; i++)
            if (distinctOrgIDs.add(orgIDs[i])) {
                List<String> wellIds = aoidDailyYieldService.getWellIdsByNew(orgIDs[i], "", localDateTime);
                List<List<String>> wellIdList = new ArrayList<>();
                Stream.iterate(0, n -> n + 1).limit(wellIds.size()).forEach(item -> {
                    wellIdList.add(wellIds.stream().skip(item * wellBatch).limit(wellBatch).collect(Collectors.toList()));
                });
                int j = 0;
                for (List<String> strings : wellIdList) {
                    calculate(strings, startDate, startTimeEveryDay);
                }
            }
    }

    @Override
    public void calculate(String wellID, LocalDateTime startDate, LocalTime startTimeEveryDay) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {

        LocalDateTime now = LocalDateTime.parse(LocalDateTime.now().format(formatterDate) + " " + time0, formatterDateTime);
        for (; Duration.between(startDate, now).toDays() >= 0; startDate = startDate.plusDays(1)) {
            String start = startDate.format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            String end = startDate.plusDays(1).format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            log.info("cal well:{}===> start:{}------end:{}", wellID, start, end);
            calculate(wellID, start, end);
        }
    }

    @Override
    public void calculate(List<String> wellIds, LocalDateTime startDate, LocalTime startTimeEveryDay) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {

        LocalDateTime now = LocalDateTime.parse(LocalDateTime.now().format(formatterDate) + " " + time0, formatterDateTime);
        for (; Duration.between(startDate, now).toDays() >= 0; startDate = startDate.plusDays(1)) {
            String start = startDate.format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            String end = startDate.plusDays(1).format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            log.info("cal well num:{}===> start:{}------end:{}", wellIds.size(), start, end);

            calculate(wellIds, start, end);
        }
    }

    @Override
    public void calculate(String wellID, String start, String end) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        calculate(wellID, LocalDateTime.parse(start, formatterDateTime), LocalDateTime.parse(end, formatterDateTime));
    }

    @Override
    public void calculate(String wellID, LocalDateTime start, LocalDateTime end) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        List<AoidSinglegtYield> lis = aoidSinglegtYieldService.getAoidSinglegtYieldInfoByWellIdAndPorDatePeriod(wellID, start, end);
        if (lis == null || lis.size() == 0) {
            log.warn("[method-calculate(String,String,String)]==>not found data for params:wellID={},start={},end={}", wellID, start, end);
            return;
        }
        calculate(lis, start, end, null);
    }

    //过滤掉非期望值
    private List<AoidSinglegtYield> unexpectedValueFilter(List<AoidSinglegtYield> list) {
        List<String> errorCondition = new ArrayList<>();
        errorCondition.add("功图错误");
        errorCondition.add("冲次异常");
        errorCondition.add("非法值过多");
        errorCondition.add("冲程异常");
        errorCondition.add("载荷差为0");
        errorCondition.add("功图存在相交区域");
        errorCondition.add("二次诊断失败");
        errorCondition.add("功图为空");
        errorCondition.add("功图点数不足10");
        for (String s : errorCondition) {
            list = list.stream().filter(x -> !x.getEczdjg().equals(s)).collect(Collectors.toList());
        }
        // 喷抽 不过滤
        List<AoidSinglegtYield> t = list.stream().filter(x -> x.getEczdjg().equals("喷抽")).collect(Collectors.toList());
        if (!t.isEmpty()) return list;
        //漏失工况 连续4条不过滤,否则过滤
        List<AoidSinglegtYield> lsList = new ArrayList<>();
        int num = 0;
        boolean flag;
        List<Integer[]> in = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            if (i == list.size() - 1) {
                if (num >= 4) {
                    Integer[] integers = {num, i};
                    in.add(integers);
                }
            }
            flag = list.get(i).getEczdjg().contains("漏失");
            if (flag) num++;
            else {
                if (num >= 4) {
                    Integer[] integers = {num, i};
                    in.add(integers);
                }
                num = 0;
            }
        }
        for (Integer[] integers : in) {
            int start = integers[1] - integers[0];
            if (integers[1] == list.size() - 1) {
                lsList.addAll(list.subList(start, list.size()));
            } else {
                lsList.addAll(list.subList(start, integers[1]));
            }
        }
        List<AoidSinglegtYield> realList = new ArrayList<>(lsList);
        // 正常和供液不足 并且 产液值 大于0.2 的被过滤出来这些不参与过滤
        List<AoidSinglegtYield> nofilter = list.stream().
                filter(x -> (x.getEczdjg().contains("正常") || x.getEczdjg().contains("供液不足"))
                        && (Objects.nonNull(x.getLiqProdDaily()) && x.getLiqProdDaily() > 0.2))
                .collect(Collectors.toList());
        realList.addAll(nofilter);
        List<AoidSinglegtYield> filter = list.stream().filter(x -> !realList.contains(x))
                .filter(x -> !x.getEczdjg().equals("漏失")).collect(Collectors.toList());
        Map<Double, List<AoidSinglegtYield>> fristList = filter.stream().filter(x -> x.getLiqProdDaily() != null).collect(Collectors.groupingBy(x -> {
            double v = x.getLiqProdDaily() / 2.5;
            return v > Math.floor(v) ? Math.floor(v) + 1 : Math.floor(v);
        }));
        List<AoidSinglegtYield> res1 = new ArrayList<>();
        for (Map.Entry<Double, List<AoidSinglegtYield>> doubleListEntry : fristList.entrySet()) {
            int size = doubleListEntry.getValue().size();
            double v = BigDecimal.valueOf((float) size / list.size()).setScale(2, RoundingMode.HALF_UP).doubleValue();
            double x = v * 100;
            if (x > 10) {
                res1.addAll(doubleListEntry.getValue());
            }
        }
        if (res1.isEmpty()) res1 = list;
        List<AoidSinglegtYield> collectDis = res1.stream().distinct().collect(Collectors.toList());
        Map<String, Integer> map = new TreeMap<>();
        Map<String, List<AoidSinglegtYield>> collect = collectDis.stream().collect(Collectors.groupingBy(AoidSinglegtYield::getZdjg));
        for (Map.Entry<String, List<AoidSinglegtYield>> stringListEntry : collect.entrySet()) {
            map.put(stringListEntry.getKey(), stringListEntry.getValue().size());
        }
        List<Integer> collect1 = new ArrayList<>(map.values());
        Collections.sort(collect1);
        double total = 0.0;
        int delMaxIndex = -1;
        for (int i = 0; i < collect1.size() - 1; i++) {
            Double item = Double.valueOf(collect1.get(i));
            total = total + item;
            if (total >= calRatio) break;
            if (total < calRatio && Math.abs(collect1.get(i + 1) - item) > 10) {
                delMaxIndex = i;
            }
        }
        List<Integer> list1 = collect1.subList(delMaxIndex + 1, collect1.size());
        List<String> res = new ArrayList<>();
        for (Integer integer : list1) {
            for (Map.Entry<String, Integer> stringIntegerEntry : map.entrySet()) {
                if (Objects.equals(stringIntegerEntry.getValue(), integer)) res.add(stringIntegerEntry.getKey());
            }
        }
        List<String> collect2 = res.stream().distinct().collect(Collectors.toList());
        List<AoidSinglegtYield> temp = res1.stream().filter(x -> collect2.contains(x.getZdjg())).collect(Collectors.toList());
        // 最终过滤结果和 不需要过滤的取并集就是最终的结果
        List<AoidSinglegtYield> listAll = realList.parallelStream().collect(Collectors.toList());
        listAll.addAll(temp.parallelStream().collect(Collectors.toList()));
        return listAll.stream().distinct().collect(Collectors.toList());
    }

    @Override
    public void calculate(List<AoidSinglegtYield> lis, LocalDateTime start, LocalDateTime end, Double time) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        List<AoidSinglegtYield> lisOriginal;
        List<AoidSinglegtYield> lisFilter;
        lisOriginal = lis;
        LocalDateTime prodStart = lis.get(0).getProdDate();
        LocalDateTime prodEnd = lis.get(lis.size() - 1).getProdDate();
        double d = 0;
        double closeTime = 0;
        for (int i = 0; i < lis.size(); i++) {
            AoidSinglegtYield yield = lis.get(i);
            if (i >= lis.size() - 1) break;
            long minutes = Duration.between(yield.getProdDate(), lis.get(i + 1).getProdDate()).toMinutes();
            if ((yield.getStroke() > 0 && yield.getFrequence() > 0 )|| !StringUtil.isBlankOrEmpty(yield.getBgt())) {
                if (closeTime <= 70) {
                    d += closeTime;
                    closeTime = 0;
                }
                d += Math.min(minutes, 60);
            } else {
                closeTime += Math.min(minutes, 60);
            }
        }
        if (Duration.between(LocalDateTime.now(), end).toMinutes() < 30 && Duration.between(prodEnd, end).toMinutes() < 70)
            d += 60;
        d = Math.min(Math.ceil(d / 60), 24);
        lisFilter = unexpectedValueFilter(lis);
        lisFilter = lisFilter.stream().sorted(Comparator.comparing(AoidSinglegtYield::getProdDate)).collect(Collectors.toList());
        AoidDailyYield re = new AoidDailyYield();
        //汇总现在是实时汇总,当天数据当天汇总
        re.setProdDate(LocalDateTime.parse(start.format(formatterDate) + " 00:00:00", formatterDateTime));
        int calNormalCount = 0;
        int strokeDataCount = 0;
        int frequenceDataCount = 0;
        int calCount = 0;
        Set<String> distinctZdjg = new HashSet<>();
        double effectiveStrokeValue1 = lis.get(0).getYxcc() == null ? 0 : lis.get(0).getYxcc();
        double effectiveStrokeSum = 0.0;
        double area1 = 0.0;
        double areaSum = 0.0;
        double area = 0.0;
        double areaRate = 0.0;

        for (AoidSinglegtYield aoidSinglegtYield : lis) {
            effectiveStrokeSum += aoidSinglegtYield.getYxcc() == null ? 0 : aoidSinglegtYield.getYxcc();
        }

        double effectiveStroke = effectiveStrokeSum / lis.size();
        double effectiveStrokeRate = Math.abs((effectiveStroke - effectiveStrokeValue1) / effectiveStrokeValue1);

        if (Objects.nonNull(lis.get(0).getArea())) {
            for (AoidSinglegtYield li : lis) {
                areaSum += Double.parseDouble(li.getArea());
            }
            area1 = Double.parseDouble(lis.get(0).getArea());
            area = areaSum / lis.size();
            areaRate = Math.abs((area - area1) / area1);
        }

        //region prod time
        Integer prodTime = null;
        Integer a2Time = null;
        Integer workDiagramTime = null;
        Integer workDiagramTimeValue = null;


        a2Time = (time == null) ? 0 : time.intValue();


        workDiagramTime = Integer.valueOf(String.valueOf(Duration.between(prodStart, prodEnd).toHours()));
        workDiagramTimeValue = workDiagramTime + 1;

        if (a2Time >= workDiagramTimeValue) {
            prodTime = a2Time;
        } else {
            prodTime = workDiagramTimeValue;
        }

        //re.setProdTime(prodTime);
        re.setProdTime((int) d);
        //准东默认如果有一张共图也是算24小时
        if (re.getProdTime()>0) re.setProdTime(24);
        //re.setProdTime(24);
        //endregion
        //设置有效/无效功图数的初始值
        re.setYxgts(0);
        re.setErrorgts(0);
        for (AoidSinglegtYield item : lisFilter) {

            //region 迭代值取法
            if (item.getCalIsNormal() == 1) {
                //一天内出现的功图种类,通过比例来判断是否当天内该井的功图混乱
                if (calNormalCount == 0) {
                    re.setLiqProdDaily(item.getLiqProdDaily());
//                    re.setWaterProdDaily(item.getWaterProdDaily());
                    re.setWaterCut(item.getHsl());
//                    re.setGasProdDaily(item.getGasProdDaily());
//                    re.setOilProdDaily(item.getOilProdDaily());
                    re.setBx(item.getBx());
                    re.setLlpl(item.getLlpl());
                    calNormalCount++;
                } else {
                    if (item.getLiqProdDaily() >= 0) {
                        re.setLiqProdDaily((re.getLiqProdDaily() * calNormalCount / (calNormalCount + 1)) + (item.getLiqProdDaily() / (calNormalCount + 1)));
//                    re.setWaterProdDaily(re.getWaterProdDaily() * calNormalCount / (calNormalCount + 1) + item.getWaterProdDaily() / (calNormalCount + 1));
                        // re.setWaterCut(re.getWaterCut() * calNormalCount / (calNormalCount + 1) + item.getHsl() / (calNormalCount + 1));
//                    re.setGasProdDaily(re.getGasProdDaily() * calNormalCount / (calNormalCount + 1) + item.getGasProdDaily() / (calNormalCount + 1));
//                    re.setOilProdDaily(re.getOilProdDaily() * calNormalCount / (calNormalCount + 1) + item.getOilProdDaily() / (calNormalCount + 1));
                        re.setBx(re.getBx() * calNormalCount / (calNormalCount + 1) + item.getBx() / (calNormalCount + 1));
                        re.setLlpl(re.getLlpl() * calNormalCount / (calNormalCount + 1) + item.getLlpl() / (calNormalCount + 1));

                        calNormalCount++;
                    }
                }
                calCount++;
            }
            //region 冲程冲次
            if (Objects.nonNull(item.getStroke()) && item.getStroke() > 0 && item.getStroke() <= 100) {
                if (strokeDataCount == 0) re.setStroke(item.getStroke());
                else
                    re.setStroke(re.getStroke() * strokeDataCount / (strokeDataCount + 1) + item.getStroke() / (strokeDataCount + 1));
                strokeDataCount++;
            }
            if (Objects.nonNull(item.getFrequence()) && item.getFrequence() > 0 && item.getFrequence() <= 100) {
                if (frequenceDataCount == 0) re.setFrequence(item.getFrequence());
                else
                    re.setFrequence(re.getFrequence() * frequenceDataCount / (frequenceDataCount + 1) + item.getFrequence() / (frequenceDataCount + 1));
                frequenceDataCount++;
            }

            //endregion
            //endregion

            //region sgt legal count


            if (Objects.nonNull(item.getEczdjg()))
                distinctZdjg.add(item.getEczdjg());
            //endregion

            //region 固定值取法 step:end
            //日志数据对应时间严格确认 -1 day;现在数据连续日期较差->会导致null问题
            //if (!start.format(formatterDate).equals(item.getProdDate().format(formatterDate))) continue;
            if (start.isBefore(item.getProdDate())) continue;
            if (Objects.nonNull(item.getWellId())) re.setWellId(item.getWellId());
            if (Objects.nonNull(item.getWellCommonName())) re.setWellCommonName(item.getWellCommonName());
            if (Objects.nonNull(item.getCreateTime())) re.setCreateTime(LocalDateTime.now());//创建时间
            if (Objects.nonNull(item.getTestLiquVolDaily()))
                re.setTestLiquVolDaily(item.getTestLiquVolDaily());//计量产液量
            if (Objects.nonNull(item.getHsl())) re.setWaterCut(item.getHsl());
            if (Objects.nonNull(item.getDym())) re.setDym(item.getDym());

            A2Basedata a2Basedata = getA2Basedata(item.getWellId(), start.format(formatterDate));
            re.setA2LiqProdDaily(a2Basedata.a2liqProdDaily);
            re.setA2OilProdDaily(a2Basedata.a2oilProdDaily);
            re.setWaterProdDaily(a2Basedata.a2waterProdDaily);
            re.setA2GasProdDaily(a2Basedata.a2gasProdDaily);
            re.setA2WaterCut(a2Basedata.a2hsl);
            re.setQyb(a2Basedata.a2qyb);
            re.setBj(a2Basedata.a2bj);
            re.setYz(a2Basedata.a2bj);
            re.setBs(a2Basedata.a2bs);
            re.setCmd(a2Basedata.a2cmd);
            re.setA2Dym(a2Basedata.a2dym);//新增A2动液面
            re.setYy(a2Basedata.a2tubingPres);//新增A2油压
            re.setTy(a2Basedata.a2casingPres);//新增A2套压
            re.setRzbz(a2Basedata.a2remarks);
            re.setYmd(a2Basedata.a2ymd);
            re.setSmd(a2Basedata.a2smd);
            re.setA2ProdTime(a2Basedata.a2prodTime);
            if (item.getParmFlag() != null && item.getParmFlag().equals("2")) re.setNotConfigure("无配置");

            String calibrationNewDescribe = null;
            String effectiveStrokeDescribe = null;
            String areaDescribe = null;
            String WorkDiagramEmptyDescribe = null;

            LocalDateTime realDate = LocalDateTime.parse(start.format(formatterDate) + " 00:00:00", formatterDateTime).minusDays(5);
            if (Objects.nonNull(item.getComparisonProdDate())) {
                if (item.getComparisonProdDate().isBefore(realDate)) {
                    calibrationNewDescribe = "需标定-";
                } else {
                    calibrationNewDescribe = "-";
                }
            }

            if (effectiveStrokeRate > 0.1) {
                effectiveStrokeDescribe = "有效冲程不稳定-";
            } else {
                effectiveStrokeDescribe = "有效冲程稳定-";
            }

            if (areaRate > 0.1) {
                areaDescribe = "面积不稳定-";
            } else {
                areaDescribe = "面积稳定-";
            }

            if (re.getLiqProdDaily() == null) {
                WorkDiagramEmptyDescribe = "功图错误";
            } else {
                WorkDiagramEmptyDescribe = "功图稳定";
            }


            String paramSummary = "(" + calibrationNewDescribe + "" + effectiveStrokeDescribe + "" + areaDescribe + "" + WorkDiagramEmptyDescribe + ")";
            re.setSummaryDescription(paramSummary);
            //endregion
        }
        if (StringUtil.isBlankOrEmpty(re.getWellId()) || StringUtil.isBlankOrEmpty(re.getWellCommonName())) return;
        for (AoidSinglegtYield item : lisOriginal) {
            if (item.getSgtIsLegal() == 1) {
                if (re.getYxgts() == null) re.setYxgts(1);
                else re.setYxgts(re.getYxgts() + 1);
            } else {
                if (re.getErrorgts() == null) re.setErrorgts(1);
                else re.setErrorgts(re.getErrorgts() + 1);
            }
        }


        if (calCount == 0) re.setBz("计算异常");
        else re.setBz(null);
        re.setZdjg(StringUtils.join(distinctZdjg.toArray(), ';'));

        if (re.getLiqProdDaily() != null && re.getWaterCut() != null) {
            //if (!StringUtil.isBlankOrEmpty(re.getBz()) && re.getBz().contains("间开")) prodTime = workDiagramTime;
            double rate = re.getProdTime() / 24d;
            re.setLiqProdDaily(Math.round(re.getLiqProdDaily() * rate * 10) / 10d);
            re.setWaterProdDaily(Math.round((re.getLiqProdDaily() * re.getWaterCut() / 10d)) / 10d);
            re.setOilProdDaily(Math.round((re.getLiqProdDaily() - re.getWaterProdDaily()) * 10) / 10d);
            if (re.getQyb() != null)
                re.setGasProdDaily(Math.round((re.getOilProdDaily() * re.getQyb()) * 10) / 10d);
        }
        if (!aoidDailyYieldService.update(re,
                new LambdaUpdateWrapper<AoidDailyYield>()
                        .eq(AoidDailyYield::getWellId, re.getWellId())
                        .eq(AoidDailyYield::getProdDate, re.getProdDate())))
            aoidDailyYieldService.save(re);

    }


    //获取A2基础数据
    public A2Basedata getA2Basedata(String wellId, String A2ProdDate) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {

        String A2ProdDateValue = A2ProdDate + " 00:00:00";

        A2Basedata a2Basedata = new A2Basedata();
        ProDataForCalProduction a2DataVolYesterday;
        ProDataForCalProduction a2DataStatusYesterday;


        String a2dataStrVolYesterday = baseDataClient.getA2VolDataForCalProduction(wellId, A2ProdDateValue);
        a2DataVolYesterday = (ProDataForCalProduction) JSONObject.parseObject(a2dataStrVolYesterday, Result.class).toFiguredOutObject(ProDataForCalProduction.class);

        String a2dataStrYesterday = baseDataClient.getA2StatusDataForCalProduction(wellId, A2ProdDateValue);
        a2DataStatusYesterday = (ProDataForCalProduction) JSONObject.parseObject(a2dataStrYesterday, Result.class).toFiguredOutObject(ProDataForCalProduction.class);


        if (a2DataVolYesterday != null) {


            a2Basedata.a2bj = (a2DataStatusYesterday.getBj() == null) ? 0 : a2DataStatusYesterday.getBj();

            a2Basedata.a2bs = (a2DataStatusYesterday.getBs() == null) ? 0 : a2DataStatusYesterday.getBs();

            a2Basedata.a2hsl = (a2DataVolYesterday.getHsl() == null) ? 0 : a2DataVolYesterday.getHsl();

            a2Basedata.a2qyb = (a2DataVolYesterday.getQyb() == null) ? 0 : a2DataVolYesterday.getQyb();

            a2Basedata.a2dym = (a2DataStatusYesterday.getDym() == null) ? 0 : a2DataStatusYesterday.getDym();

            a2Basedata.a2ymd = (a2DataVolYesterday.getYmd() == null) ? 0 : a2DataVolYesterday.getYmd();

            a2Basedata.a2smd = (a2DataVolYesterday.getSmd() == null) ? 0 : a2DataVolYesterday.getSmd();


            a2Basedata.a2backPres = (a2DataStatusYesterday.getBackPres() == null) ? 0 : a2DataStatusYesterday.getBackPres();
            a2Basedata.a2tubingPres = (a2DataStatusYesterday.getTubingPres() == null) ? 0 : a2DataStatusYesterday.getTubingPres();
            a2Basedata.a2casingPres = (a2DataStatusYesterday.getCasingPres() == null) ? 0 : a2DataStatusYesterday.getCasingPres();


            if (Objects.nonNull(a2DataStatusYesterday.getBs()) && Objects.nonNull(a2DataStatusYesterday.getDym())) {
                a2Basedata.a2cmd = Math.abs(a2DataStatusYesterday.getBs().doubleValue() - a2DataStatusYesterday.getDym().doubleValue());
            }


            a2Basedata.a2liqProdDaily = (a2DataVolYesterday.getLiqProdDaily() == null) ? 0 : a2DataVolYesterday.getLiqProdDaily();
            a2Basedata.a2oilProdDaily = (a2DataVolYesterday.getOilProdDaily() == null) ? 0 : a2DataVolYesterday.getOilProdDaily();
            a2Basedata.a2waterProdDaily = (a2DataVolYesterday.getWaterProdDaily() == null) ? 0 : a2DataVolYesterday.getWaterProdDaily();
            a2Basedata.a2gasProdDaily = (a2DataVolYesterday.getGasProdDaily() == null) ? 0 : a2DataVolYesterday.getGasProdDaily();

            a2Basedata.a2remarks = a2DataVolYesterday.getRemarks();
            a2Basedata.a2prodTime = a2DataVolYesterday.getProdTime();
        }

        return a2Basedata;
    }


    //A2基础数据类
    public class A2Basedata {
        private Double a2bj;

        private Double a2bs;

        private Double a2hsl;

        private Double a2qyb;

        private Double a2dym;

        private Double a2ymd;

        private Double a2smd;

        private String a2proDate;


        private Double a2backPres;
        private Double a2tubingPres;
        private Double a2casingPres;


        private Double a2cmd;

        private Double a2liqProdDaily;
        private Double a2oilProdDaily;
        private Double a2waterProdDaily;
        private Double a2gasProdDaily;

        private String a2remarks;
        private Integer a2prodTime;


    }

    @Override
    public void calculate(List<String> wellIds, String start, String end) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        LocalDateTime useDate;
        LocalDateTime startDate = DateTimeUtil.formatDateTime(start);
        LocalDateTime endDate = DateTimeUtil.formatDateTime(end);
        if (LocalTime.now().isAfter(startDate.toLocalTime())) {
            useDate = startDate.plusDays(1);
        } else {
            useDate = startDate;
            startDate = startDate.minusDays(1);
            endDate = endDate.minusDays(1);
        }
        List<AoidSinglegtYield> lis = aoidSinglegtYieldService.getAoidSinglegtYieldInfoByWellIdAndPorDatePeriodNew(wellIds, startDate, endDate);
        if (lis != null && lis.size() > 0) {
            Collection<List<AoidSinglegtYield>> values = lis.stream().collect(Collectors.groupingBy(AoidSinglegtYield::getWellId)).values();
            for (List<AoidSinglegtYield> value : values) {
                if (value != null && value.size() > 0) {
                    Double time = null;
                    if (TimingSummaryJob.threadLocal.get()) {
                        String a2VolDataForCalProduction = baseDataClient.getA2VolDataForCalProduction(value.get(0).getWellId(), start);//可能用start 或 start+1
                        Object data = JSONObject.parseObject(a2VolDataForCalProduction, Result.class).getData();
                        if (data != null) {
                            HashMap hashMap = JSONObject.parseObject(data.toString(), HashMap.class);
                            if (hashMap != null && hashMap.size() > 0) {
                                BigDecimal prodTime = (BigDecimal) hashMap.get("prodTime");
                                time = Double.valueOf(prodTime.toString());
                            }
                        }
                    }
                    calculate(value, useDate, endDate, time);
                }
            }
        }
    }
}
