package com.gct.aoid.utils;

import com.alibaba.fastjson.JSONObject;
import com.gct.aoid.entity.log.LogFile;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * class name: LogFile
 *
 * @author gxt
 * @version 1.0
 * @since 2021/8/11 下午5:57 周三
 */
public class LogFileUtils {


    public static List<LogFile> getLogFile() {
        try {
            File jsonFile = createLogFile();
            BufferedReader fileReader = new BufferedReader(new FileReader(jsonFile));
            Reader reader = new InputStreamReader(new FileInputStream(jsonFile), "utf-8");
            int ch = 0;
            long count = Files.lines(Paths.get(jsonFile.getAbsolutePath())).count();
            StringBuffer sb = new StringBuffer();
            while ((ch = reader.read()) != -1) {
                sb.append((char) ch);
            }

            List<String> strings = Files.readAllLines(Paths.get(jsonFile.getAbsolutePath()));
            if (strings.size() == 10) {
                List<LogFile> list = new ArrayList<>();
                strings.forEach(x -> list.add(JSONObject.parseObject(x, LogFile.class)));
                List<LogFile> collect = list.stream().sorted(Comparator.comparing(LogFile::getProdDate)).collect(Collectors.toList()).subList(5, 10);
                removeLogFile(jsonFile.getAbsolutePath(), collect);
            }
            fileReader.close();
            reader.close();
            List<LogFile> list = new ArrayList<>();
            strings.forEach(x -> list.add(JSONObject.parseObject(x, LogFile.class)));
            return list;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static void writeLogFile(JSONObject msg) {
        String msgJson = JSONObject.toJSONString(msg);
        File logFile = createLogFile();
        try {
            File file = new File(logFile.getAbsolutePath());
            BufferedWriter out = null;
            if (!file.exists()) {
                file.createNewFile();
                out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));

            } else {
                out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file, true)));
            }
            out.write(msgJson + "\n");
            out.flush();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void removeLogFile(String fileName, List<LogFile> msg) {
        List<String> list = new ArrayList<>();
        msg.forEach(x -> list.add(JSONObject.toJSONString(x)));
        File file = new File(fileName);
        if (file.exists()) {
            file.delete();
            list.forEach(x -> writeLogFile(JSONObject.parseObject(x)));
        }
    }

    public static File createLogFile() {
        LocalDate now = LocalDate.now();
        Map<String, String> env = System.getenv();
        String fileName = env.get("HOME") + "/logs/production-cal-app/" + now + ".log";
        File jsonFile = new File(fileName);
        if (!jsonFile.getParentFile().exists()) jsonFile.getParentFile().mkdirs();
        if (!jsonFile.exists()) {
            try {
                jsonFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return jsonFile;
    }

    public static boolean isRunning(LocalDateTime localDateTime) throws IOException {

        File logFile = createLogFile();
        LocalDateTime localDateTime1 = LocalDateTime.ofInstant(Files.getLastModifiedTime(Paths.get(logFile.getAbsolutePath())).toInstant(), ZoneId.systemDefault());
        long subHours = Duration.between(localDateTime, localDateTime1).toHours();
        if (subHours >= 2) {
            return false;
        }
        return true;
    }

    public static String getLastModifiedTime() throws IOException {
        try {
            File logFile = createLogFile();
            LocalDateTime localDateTime = LocalDateTime.ofInstant(Files.getLastModifiedTime(Paths.get(logFile.getAbsolutePath())).toInstant(), ZoneId.systemDefault());
            return localDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
}
